@@ -196,7 +196,7 @@ flag_options_xref StyleDataFlags[] = {
 };
 
 TermDef       DatabaseTerms[] = {
-	{TF_NAMED_SUBCONFIG | TF_NO_MYNAME_PREPENDING, "Style", 5, TT_QUOTED_TEXT, DATABASE_STYLE_ID, &StyleSyntax}
+	{TF_NAMED | TF_NO_MYNAME_PREPENDING, "Style", 5, TT_QUOTED_TEXT, DATABASE_STYLE_ID, &StyleSyntax}
 	,
 	{0, NULL, 0, 0, 0}
 };
@@ -542,7 +542,6 @@ ParseDatabaseOptions (const char *filename, char *myname)
 	item.memory = NULL;
 	PrintConfigReader (ConfigReader);
 	ParseConfig (ConfigReader, &Storage);
-	PrintFreeStorage (Storage);
 
 	/* getting rid of all the crap first */
 	StorageCleanUp (&Storage, &more_stuff, CF_DISABLED_OPTION);
