@@ -138,9 +138,36 @@ as_image_loader_func as_image_file_loaders[ASIT_Unknown] =
 	tiff2ASImage,
 	xml2ASImage ,
 	NULL,
+	tga2ASImage,
+	NULL,
 	NULL,
 	NULL
 };
+
+const char *as_image_file_type_names[ASIT_Unknown+1] =
+{
+	"XPM" ,
+	"Z-compressed XPM" ,
+	"GZ-compressed XPM" ,
+	"PNG" ,
+	"JPEG",
+	"GIMP Xcf" ,
+	"PPM" ,
+	"PNM" ,
+	"MS Windows Bitmap" ,
+	"MS Windows Icon" ,
+	"MS Windows Cursor" ,
+	"GIF" ,
+	"TIFF",
+	"AfterStep XML" ,
+	"XBM",
+	"Targa",
+	"PCX",
+	"HTML",
+	"XML",
+	"Unknown"
+};
+
  
 ASImage *
 file2ASImage_extra( const char *file, ASImageImportParams *iparams )
@@ -298,7 +325,7 @@ file2pixmap(ASVisual *asv, Window root, const char *realfilename, Pixmap *mask_o
 	}
 	if( mask_out )
 	{
-		if( *mask_out )
+		if( *mask_out && asv )
 			XFreePixmap( asv->dpy, *mask_out );
 		*mask_out = mask ;
 	}
@@ -329,6 +356,186 @@ get_asimage( ASImageManager* imageman, const char *file, ASFlagType what, unsign
 	return im;
 }
 
+ASImageListEntry * 
+ref_asimage_list_entry( ASImageListEntry *entry )
+{
+	if( entry ) 
+	{
+		if( IS_ASIMAGE_LIST_ENTRY(entry) )
+			++(entry->ref_count);
+		else
+			entry = NULL ; 
+	}
+	return entry;
+}
+	 
+ASImageListEntry *
+unref_asimage_list_entry( ASImageListEntry *entry )
+{
+	if( entry ) 
+	{	
+		if( IS_ASIMAGE_LIST_ENTRY(entry) )
+		{
+			--(entry->ref_count);
+			if( entry->ref_count  <= 0 )
+			{
+				ASImageListEntry *prev = entry->prev ; 
+				ASImageListEntry *next = entry->next ; 
+				if( !IS_ASIMAGE_LIST_ENTRY(prev) )
+					prev = NULL ; 
+				if( !IS_ASIMAGE_LIST_ENTRY(next) )
+					next = NULL ; 
+				if( prev ) 
+					prev->next = next ; 
+				if( next ) 
+					next->prev = prev ; 
+
+				if( entry->preview ) 
+					safe_asimage_destroy( entry->preview );
+				if( entry->name )
+					free( entry->name );
+				if( entry->fullfilename )
+					free( entry->fullfilename );
+				if( entry->buffer ) 
+					destroy_asimage_list_entry_buffer( &(entry->buffer) );
+				memset( entry, 0x00, sizeof(ASImageListEntry));
+				free( entry );
+				entry = NULL ; 
+			}	 
+		}else
+			entry = NULL ;
+	}
+	return entry;
+}	 
+
+ASImageListEntry *
+create_asimage_list_entry()
+{
+	ASImageListEntry *entry = safecalloc( 1, sizeof(ASImageListEntry));
+	entry->ref_count = 1 ; 
+	entry->magic = MAGIC_ASIMAGE_LIST_ENTRY ; 
+	return entry;
+}
+
+void
+destroy_asimage_list( ASImageListEntry **plist )
+{
+	if( plist )
+	{		   
+		ASImageListEntry *curr = *plist ;
+		while( IS_ASIMAGE_LIST_ENTRY(curr) )
+		{	
+			ASImageListEntry *to_delete = curr ; 
+			curr = curr->next ;
+		 	unref_asimage_list_entry( to_delete );
+		}
+		*plist = NULL ;
+	}
+}
+
+void destroy_asimage_list_entry_buffer( ASImageListEntryBuffer **pbuffer )
+{
+	if( pbuffer && *pbuffer ) 
+	{		 
+		if( (*pbuffer)->data ) 
+			free( (*pbuffer)->data ) ;
+		free( *pbuffer );
+		*pbuffer = NULL ;
+	}
+}	 
+
+struct ASImageListAuxData
+{
+	ASImageListEntry **pcurr;
+	ASImageListEntry *last ;
+	ASFlagType preview_type ;
+	unsigned int preview_width ;
+	unsigned int preview_height ;
+	unsigned int preview_compression ;
+	ASVisual *asv;
+};
+
+Bool 
+direntry2ASImageListEntry( const char *fname, const char *fullname, 
+						   struct stat *stat_info, void *aux_data)
+{
+	struct ASImageListAuxData *data = (struct ASImageListAuxData*)aux_data;
+	ASImageFileTypes file_type ;
+	ASImageListEntry *curr ;
+	   	
+	if (S_ISDIR (stat_info->st_mode))
+		return False;
+	
+	file_type = check_image_type( fullname );
+	if( file_type != ASIT_Unknown && as_image_file_loaders[file_type] == NULL )
+		file_type = ASIT_Unknown ;
+
+	curr = create_asimage_list_entry();
+	*(data->pcurr) = curr ; 
+	if( data->last )
+		data->last->next = curr ;
+	curr->prev = data->last ;
+	data->last = curr ;
+	data->pcurr = &(data->last->next);
+
+	curr->name = mystrdup( fname );
+	curr->fullfilename = mystrdup(fullname);
+	curr->type = file_type ;
+   	curr->d_mode = stat_info->st_mode;
+	curr->d_mtime = stat_info->st_mtime;
+	curr->d_size  = stat_info->st_size;
+
+	if( curr->type != ASIT_Unknown && data->preview_type != 0 )
+	{
+		ASImageImportParams iparams = {0} ;
+		ASImage *im = as_image_file_loaders[file_type](fullname, &iparams);
+		if( im )
+		{
+			int scale_width = im->width ;
+			int scale_height = im->height ;
+			int tile_width = im->width ;
+			int tile_height = im->height ;
+
+			if( data->preview_width > 0 )
+			{
+				if( get_flags( data->preview_type, SCALE_PREVIEW_H ) )
+					scale_width = data->preview_width ;
+				else
+					tile_width = data->preview_width ;
+			}
+			if( data->preview_height > 0 )
+			{
+				if( get_flags( data->preview_type, SCALE_PREVIEW_V ) )
+					scale_height = data->preview_height ;
+				else
+					tile_height = data->preview_height ;
+			}
+			if( scale_width != im->width || scale_height != im->height )
+			{
+				ASImage *tmp = scale_asimage( data->asv, im, scale_width, scale_height, ASA_ASImage, data->preview_compression, ASIMAGE_QUALITY_DEFAULT );
+				if( tmp != NULL )
+				{
+					destroy_asimage( &im );
+					im = tmp ;
+				}
+			}
+			if( tile_width != im->width || tile_height != im->height )
+			{
+				ASImage *tmp = tile_asimage( data->asv, im, 0, 0, tile_width, tile_height, TINT_NONE, ASA_ASImage, data->preview_compression, ASIMAGE_QUALITY_DEFAULT );
+				if( tmp != NULL )
+				{
+					destroy_asimage( &im );
+					im = tmp ;
+				}
+			}
+		}
+
+		curr->preview = im ;
+	}
+	return True;
+}
+
+
 ASImageListEntry *
 get_asimage_list( ASVisual *asv, const char *dir,
 	              ASFlagType preview_type, double gamma,
@@ -339,95 +546,22 @@ get_asimage_list( ASVisual *asv, const char *dir,
 {
 	ASImageListEntry *im_list = NULL ;
 #ifndef _WIN32
-	ASImageListEntry **curr = &im_list, *last = NULL ;
-	struct direntry  **list;
-	int n, i, count = 0;
-	int dir_name_len ;
-
-
+	struct ASImageListAuxData aux_data ; 
+	int count ; 
+	
+	aux_data.pcurr = &im_list;
+	aux_data.last = NULL;
+	aux_data.preview_type = preview_type;
+	aux_data.preview_width = preview_width;
+	aux_data.preview_height = preview_height;
+	aux_data.preview_compression  = preview_compression;
+	aux_data.asv = asv ; 
+	
+	
 	if( asv == NULL || dir == NULL )
 		return NULL ;
 
-	dir_name_len = strlen(dir);
-	n = my_scandir ((char*)dir, &list, select, NULL);
-	if( n > 0 )
-	{
-		for (i = 0; i < n; i++)
-		{
-			if (!S_ISDIR (list[i]->d_mode))
-			{
-				ASImageFileTypes file_type ;
-				char *realfilename ;
-
-				realfilename = safemalloc( dir_name_len + 1 + strlen(list[i]->d_name)+1);
-				sprintf( realfilename, "%s/%s", dir, list[i]->d_name );
-				file_type = check_image_type( realfilename );
-				if( file_type != ASIT_Unknown && as_image_file_loaders[file_type] == NULL )
-					file_type = ASIT_Unknown ;
-
-				++count ;
-				*curr = safecalloc( 1, sizeof(ASImageListEntry) );
-				if( last )
-					last->next = *curr ;
-				last = *curr ;
-				curr = &(last->next);
-
-				last->name = mystrdup( list[i]->d_name );
-				last->fullfilename = realfilename ;
-				last->type = file_type ;
-
-				if( last->type != ASIT_Unknown && preview_type != 0 )
-				{
-					ASImageImportParams iparams = {0} ;
-					ASImage *im = as_image_file_loaders[file_type](realfilename, &iparams);
-					if( im )
-					{
-						int scale_width = im->width ;
-						int scale_height = im->height ;
-						int tile_width = im->width ;
-						int tile_height = im->height ;
-
-						if( preview_width > 0 )
-						{
-							if( get_flags( preview_type, SCALE_PREVIEW_H ) )
-								scale_width = preview_width ;
-							else
-								tile_width = preview_width ;
-						}
-						if( preview_height > 0 )
-						{
-							if( get_flags( preview_type, SCALE_PREVIEW_V ) )
-								scale_height = preview_height ;
-							else
-								tile_height = preview_height ;
-						}
-						if( scale_width != im->width || scale_height != im->height )
-						{
-							ASImage *tmp = scale_asimage( asv, im, scale_width, scale_height, ASA_ASImage, preview_compression, ASIMAGE_QUALITY_DEFAULT );
-							if( tmp != NULL )
-							{
-								destroy_asimage( &im );
-								im = tmp ;
-							}
-						}
-						if( tile_width != im->width || tile_height != im->height )
-						{
-							ASImage *tmp = tile_asimage( asv, im, 0, 0, tile_width, tile_height, TINT_NONE, ASA_ASImage, preview_compression, ASIMAGE_QUALITY_DEFAULT );
-							if( tmp != NULL )
-							{
-								destroy_asimage( &im );
-								im = tmp ;
-							}
-						}
-					}
-
-			  		last->preview = im ;
-				}
-			}
-			free( list[i] );
-		}
-		free (list);
-	}
+	count = my_scandir_ext ((char*)dir, select, direntry2ASImageListEntry, &aux_data);
 
 	if( count_ret )
 		*count_ret = count ;
@@ -435,6 +569,79 @@ get_asimage_list( ASVisual *asv, const char *dir,
 	return im_list;
 }
 
+char *format_asimage_list_entry_details( ASImageListEntry *entry, Bool vertical )
+{
+	char *details_text ;
+
+	if( entry ) 
+	{	
+		int type = (entry->type>ASIT_Unknown)?ASIT_Unknown:entry->type ; 
+		details_text = safemalloc(128);
+		if( entry->preview ) 
+			sprintf( details_text, vertical?"File type: %s\nSize %dx%d":"File type: %s; Size %dx%d", as_image_file_type_names[type], entry->preview->width, entry->preview->height ); 	  
+		else 
+			sprintf( details_text, "File type: %s", as_image_file_type_names[type]);
+	}else
+		details_text = mystrdup("");		   
+	return details_text;
+}	 
+
+Bool 
+load_asimage_list_entry_data( ASImageListEntry *entry, size_t max_bytes )
+{
+	char * new_buffer ; 
+	size_t new_buffer_size ;
+	FILE *fp;
+	Bool binary = False ; 
+	if( entry == NULL ) 
+		return False;
+	if( entry->buffer == NULL ) 
+		entry->buffer = safecalloc( 1, sizeof(ASImageListEntryBuffer) );
+	if( entry->buffer->size == entry->d_size || entry->buffer->size >= max_bytes )
+		return True;
+	new_buffer_size = min( max_bytes, (size_t)entry->d_size ); 
+	new_buffer = malloc( new_buffer_size );
+	if( new_buffer == NULL ) 
+		return False ;
+	if( entry->buffer->size > 0 ) 
+	{	
+		memcpy( new_buffer, entry->buffer->data, entry->buffer->size ) ;
+		free( entry->buffer->data );
+	}
+	entry->buffer->data = new_buffer ; 
+	/* TODO read new_buffer_size - entry->buffer_size bytes into the end of the buffer */
+	fp = fopen(entry->fullfilename, "rb");
+	if ( fp != NULL ) 
+	{
+		int len = new_buffer_size - entry->buffer->size ;
+		if( entry->buffer->size > 0 ) 
+			fseek( fp, entry->buffer->size, SEEK_SET );
+		len = fread(entry->buffer->data, 1, len, fp);
+		if( len > 0 ) 
+			entry->buffer->size += len ;
+		fclose(fp);
+	}
+
+	if( entry->type == ASIT_Unknown ) 
+	{
+		int i = entry->buffer->size ; 
+		register char *ptr = entry->buffer->data ;
+		while ( --i >= 0 )	
+			if( !isprint(ptr[i]) && ptr[i] != '\n'&& ptr[i] != '\r'&& ptr[i] != '\t' )	
+				break;
+		binary = (i >= 0);				
+	}else
+		binary = (entry->type != ASIT_Xpm  && entry->type != ASIT_XMLScript &&
+			  	  entry->type != ASIT_HTML && entry->type != ASIT_XML ); 
+	if( binary ) 
+		set_flags( entry->buffer->flags, ASILEB_Binary );
+   	else
+		clear_flags( entry->buffer->flags, ASILEB_Binary );
+	 
+
+
+	return True;
+}
 
 /***********************************************************************************/
 /* Some helper functions :                                                         */
@@ -489,12 +696,15 @@ check_image_type( const char *realfilename )
 {
 	int filename_len = strlen( realfilename );
 #define FILE_HEADER_SIZE	512
-	CARD8 head[FILE_HEADER_SIZE+1] ;
+	char head[FILE_HEADER_SIZE+1] ;
 	int bytes_in = 0 ;
 	FILE *fp ;
 	ASImageFileTypes type = ASIT_Unknown ;
 	/* lets check if we have compressed xpm file : */
-	if( filename_len > 7 && mystrncasecmp( realfilename+filename_len-7, ".xpm.gz", 7 ) == 0 )
+	if( filename_len > 5 && (mystrncasecmp( realfilename+filename_len-5, ".html", 5 ) == 0 || 
+							 mystrncasecmp( realfilename+filename_len-4, ".htm", 4 ) == 0 ))
+		type = ASIT_HTML;
+	else if( filename_len > 7 && mystrncasecmp( realfilename+filename_len-7, ".xpm.gz", 7 ) == 0 )
 		type = ASIT_GZCompressedXpm;
 	else if( filename_len > 6 && mystrncasecmp( realfilename+filename_len-6, ".xpm.Z", 6 ) == 0 )
 		type = ASIT_ZCompressedXpm ;
@@ -508,7 +718,7 @@ check_image_type( const char *realfilename )
  */
 		if( bytes_in > 3 )
 		{
-			if( head[0] == 0xff && head[1] == 0xd8 && head[2] == 0xff)
+			if( (CARD8)head[0] == 0xff && (CARD8)head[1] == 0xd8 && (CARD8)head[2] == 0xff)
 				type = ASIT_Jpeg;
 			else if (strstr ((char *)&(head[0]), "XPM") != NULL)
 				type =  ASIT_Xpm;
@@ -531,47 +741,71 @@ check_image_type( const char *realfilename )
 						 mystrncasecmp(realfilename+filename_len-4, ".ICO", 4)==0) )
 				type = ASIT_Cur;
 		}
+		if( type == ASIT_Unknown && bytes_in  > 6 )
+		{
+			if( mystrncasecmp( head, "<HTML>", 6 ) == 0 )
+				type = ASIT_HTML;	
+		}	 
 		if( type == ASIT_Unknown && bytes_in  > 8 )
 		{
-			if( strncmp((char *)&(head[0]), XCF_SIGNATURE, (size_t) XCF_SIGNATURE_LEN) == 0)
+			if( strncmp(&(head[0]), XCF_SIGNATURE, (size_t) XCF_SIGNATURE_LEN) == 0)
 				type = ASIT_Xcf;
 	   		else if (head[0] == 0 && head[1] == 0 &&
 			    	 head[2] == 2 && head[3] == 0 && head[4] == 0 && head[5] == 0 && head[6] == 0 && head[7] == 0)
 				type = ASIT_Targa;
-			else if (strncmp ((char *)&(head[0]), "#define", (size_t) 7) == 0)
+			else if (strncmp (&(head[0]), "#define", (size_t) 7) == 0)
 				type = ASIT_Xbm;
 			else
 			{/* the nastiest check - for XML files : */
-				register int i ;
+				int i ;
 
 				type = ASIT_XMLScript ;
+				for( i = 0 ; i < bytes_in ; ++i ) if( !isspace(head[i]) ) break;
 				while( bytes_in > 0 && type == ASIT_XMLScript )
 				{
-					for( i = 0 ; i < bytes_in ; ++i ) if( !isspace(head[i]) ) break;
 					if( i >= bytes_in )
+					{	
 						bytes_in = fread( &(head[0]), sizeof(CARD8), FILE_HEADER_SIZE, fp );
+						for( i = 0 ; i < bytes_in ; ++i ) if( !isspace(head[i]) ) break;
+					}
 					else if( head[i] != '<' )
 						type = ASIT_Unknown ;
-					else
-						break ;
-				}
-				while( bytes_in > 0 && type == ASIT_XMLScript )
-				{
-					for( i = 0 ; i < bytes_in ; ++i )
-						if( !isspace(head[i]) )
+					else if( mystrncasecmp( &(head[i]), "<!DOCTYPE ", 10 ) == 0 ) 
+					{	
+						type = ASIT_XML ;
+						for( i += 9 ; i < bytes_in ; ++i ) if( !isspace(head[i]) ) break;
+						if( i < bytes_in ) 
 						{
-							if( !isprint(head[i]) )
+					 		if( mystrncasecmp( &(head[i]), "afterstep-image-xml", 19 ) == 0 ) 			
 							{
-								type = ASIT_Unknown ;
-								break ;
-							}else if( head[i] == '>' )
+								i += 19 ;	  
+								type = ASIT_XMLScript ;
+							}
+						}	 
+					}else
+					{
+						while( bytes_in > 0 && type == ASIT_XMLScript )
+						{
+							while( ++i < bytes_in )
+								if( !isspace(head[i]) )
+								{
+									if( !isprint(head[i]) )
+									{
+										type = ASIT_Unknown ;
+										break ;
+									}else if( head[i] == '>' )
+										break ;
+								}
+
+							if( i >= bytes_in )
+							{	
+								bytes_in = fread( &(head[0]), sizeof(CARD8), FILE_HEADER_SIZE, fp );
+								i = 0 ; 
+							}else
 								break ;
 						}
-
-					if( i >= bytes_in )
-						bytes_in = fread( &(head[0]), sizeof(CARD8), FILE_HEADER_SIZE, fp );
-					else
-						break ;
+						break;
+					}	
 				}
 			}
 		}
@@ -817,7 +1051,7 @@ png2ASImage( const char * path, ASImageImportParams *params )
 						raw2scanline( row_pointers[y], &buf, NULL, buf.width, grayscale, do_alpha );
 						im->channels[IC_RED][y] = store_data( NULL, (CARD8*)buf.red, buf.width*4, rgb_flags, 0);
 					}else
-						im->channels[IC_RED][y] = store_data( NULL, row_pointers[y], buf.width, rgb_flags, 0);
+						im->channels[IC_RED][y] = store_data( NULL, row_pointers[y], im->width, rgb_flags, 0);
 					
 					if( grayscale ) 
 					{	
@@ -1774,3 +2008,290 @@ xml2ASImage( const char *path, ASImageImportParams *params )
 	SHOW_TIME("image loading",started);
 	return im ;
 }
+/*************************************************************************/
+/* Targa Image format - some stuff borrowed from the GIMP.
+ *************************************************************************/
+typedef struct ASTGAHeader
+{
+	CARD8 IDLength ;
+	CARD8 ColorMapType;
+#define TGA_NoImageData			0
+#define TGA_ColormappedImage	1
+#define TGA_TrueColorImage		2
+#define TGA_BWImage				3
+#define TGA_RLEColormappedImage		9
+#define TGA_RLETrueColorImage		10
+#define TGA_RLEBWImage				11
+	CARD8 ImageType;
+	struct 
+	{
+		CARD16 FirstEntryIndex ;
+		CARD16 ColorMapLength ;  /* number of entries */ 
+		CARD8  ColorMapEntrySize ;  /* number of bits per entry */ 
+	}ColormapSpec;
+	struct
+	{		
+		CARD16 XOrigin;
+		CARD16 YOrigin;
+		CARD16 Width;
+		CARD16 Height;
+		CARD8  Depth;
+#define TGA_LeftToRight		(0x01<<4)
+#define TGA_TopToBottom		(0x01<<5)
+		CARD8  Descriptor;
+	}ImageSpec;
+
+}ASTGAHeader;
+
+typedef struct ASTGAColorMap
+{
+	int bytes_per_entry;
+	int bytes_total ; 
+	CARD8 *data ; 
+}ASTGAColorMap;
+
+typedef struct ASTGAImageData
+{
+	int bytes_per_pixel;
+	int image_size;
+	int bytes_total ; 
+	CARD8 *data ; 
+}ASTGAImageData;
+
+static Bool load_tga_colormapped(FILE *infile, ASTGAHeader *tga, ASTGAColorMap *cmap, ASScanline *buf, CARD8 *read_buf, CARD8 *gamma_table )
+{
+		
+	return True;
+}
+
+static Bool load_tga_truecolor(FILE *infile, ASTGAHeader *tga, ASTGAColorMap *cmap, ASScanline *buf, CARD8 *read_buf, CARD8 *gamma_table )
+{
+	CARD32 *a = buf->alpha ;
+	CARD32 *r = buf->red ;
+	CARD32 *g = buf->green ;
+	CARD32 *b = buf->blue ;
+	int bpp = (tga->ImageSpec.Depth+7)/8;
+	int bpl = buf->width*bpp;
+	if( fread( read_buf, 1, bpl, infile ) != bpl ) 		   
+		return False;
+	if( bpp == 3 ) 
+	{	
+		int i;
+		if( gamma_table )
+			for( i = 0 ; i < buf->width ; ++i ) 
+			{
+				b[i] = gamma_table[*(read_buf++)];	
+				g[i] = gamma_table[*(read_buf++)];	  
+				r[i] = gamma_table[*(read_buf++)];	  
+			}	 
+		else
+			for( i = 0 ; i < buf->width ; ++i ) 
+			{
+				b[i] = *(read_buf++);	
+				g[i] = *(read_buf++);	  
+				r[i] = *(read_buf++);	  
+			}	 
+		set_flags( buf->flags, SCL_DO_RED|SCL_DO_GREEN|SCL_DO_BLUE );
+	}else if( bpp == 4 )
+	{
+		int i;
+		for( i = 0 ; i < buf->width ; ++i ) 
+		{
+			b[i] = *(read_buf++);	
+			g[i] = *(read_buf++);	  
+			r[i] = *(read_buf++);	  
+			a[i] = *(read_buf++);	  
+		}	 
+		set_flags( buf->flags, SCL_DO_RED|SCL_DO_GREEN|SCL_DO_BLUE|SCL_DO_ALPHA );
+	}	 
+
+	return True;
+}
+
+static Bool load_tga_bw(FILE *infile, ASTGAHeader *tga, ASTGAColorMap *cmap, ASScanline *buf, CARD8 *read_buf, CARD8 *gamma_table )
+{
+		
+	return True;
+}
+
+static Bool load_tga_rle_colormapped(FILE *infile, ASTGAHeader *tga, ASTGAColorMap *cmap, ASScanline *buf, CARD8 *read_buf, CARD8 *gamma_table )
+{
+		
+	return True;
+}
+
+static Bool load_tga_rle_truecolor(FILE *infile, ASTGAHeader *tga, ASTGAColorMap *cmap, ASScanline *buf, CARD8 *read_buf, CARD8 *gamma_table )
+{
+		
+	return True;
+}
+
+static Bool load_tga_rle_bw(FILE *infile, ASTGAHeader *tga, ASTGAColorMap *cmap, ASScanline *buf, CARD8 *read_buf, CARD8 *gamma_table )
+{
+		
+	return True;
+}
+
+
+
+ASImage *
+tga2ASImage( const char * path, ASImageImportParams *params )
+{
+	ASImage *im = NULL ;
+	/* More stuff */
+	FILE         *infile;					   /* source file */
+	ASTGAHeader   tga;
+	ASTGAColorMap *cmap = NULL ;
+	int width = 1, height = 1;
+	START_TIME(started);
+
+
+	if ((infile = open_image_file(path)) == NULL)
+		return NULL;
+	if( fread( &tga, 1, 3, infile ) == 3 ) 
+	if( fread( &tga.ColormapSpec, 1, 5, infile ) == 5 ) 
+	if( fread( &tga.ImageSpec, 1, 10, infile ) == 10 ) 
+	{
+		Bool success = True ;
+		Bool (*load_row_func)(FILE *infile, ASTGAHeader *tga, ASTGAColorMap *cmap, ASScanline *buf, CARD8 *read_buf, CARD8 *gamma_table );
+
+		if( tga.IDLength > 0 ) 
+			success = (fseek( infile, tga.IDLength, SEEK_CUR )==0);
+		if( success && tga.ColorMapType != 0 ) 
+		{
+			cmap = safecalloc( 1, sizeof(ASTGAColorMap));
+			cmap->bytes_per_entry = (tga.ColormapSpec.ColorMapEntrySize+7)/8;
+			cmap->bytes_total = cmap->bytes_per_entry*tga.ColormapSpec.ColorMapLength; 
+			cmap->data = safemalloc( cmap->bytes_total);
+			success = ( fread( cmap->data, 1, cmap->bytes_total, infile ) == cmap->bytes_total );
+		}else if( tga.ImageSpec.Depth != 24 && tga.ImageSpec.Depth != 32 )
+			success = False ;
+	 
+		if( success ) 
+		{
+			success = False;
+			if( tga.ImageType != TGA_NoImageData )
+			{	
+				width = tga.ImageSpec.Width ; 
+				height = tga.ImageSpec.Height ; 
+				if( width < MAX_IMPORT_IMAGE_SIZE && height < MAX_IMPORT_IMAGE_SIZE )
+					success = True;
+			}
+		}
+		switch( tga.ImageType ) 
+		{
+			case TGA_ColormappedImage	:load_row_func = load_tga_colormapped ; break ;
+			case TGA_TrueColorImage		:load_row_func = load_tga_truecolor ; break ;
+			case TGA_BWImage			:load_row_func = load_tga_bw ; break ;
+			case TGA_RLEColormappedImage:load_row_func = load_tga_rle_colormapped ; break ;
+			case TGA_RLETrueColorImage	:load_row_func = load_tga_rle_truecolor ; break ;
+			case TGA_RLEBWImage			:load_row_func = load_tga_rle_bw ; break ;
+			default:
+				load_row_func = NULL ;
+		}	 
+		
+		if( success && load_row_func != NULL ) 
+		{	
+			ASImageOutput  *imout ;
+			im = create_asimage( width, height, params->compression );
+			if((imout = start_image_output( NULL, im, ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT)) == NULL )
+			{
+        		destroy_asimage( &im );
+				success = False;
+			}else
+			{	
+				ASScanline    buf;
+				int y ;
+				CARD8 *read_buf = safemalloc( width*4*2 ); 
+				prepare_scanline( im->width, 0, &buf, True );
+				if( !get_flags( tga.ImageSpec.Descriptor, TGA_TopToBottom ) )			
+					toggle_image_output_direction( imout );
+				for( y = 0 ; y < height ; ++y ) 
+				{	
+					if( !load_row_func( infile, &tga, cmap, &buf, read_buf, params->gamma_table ) )
+						break;
+					imout->output_image_scanline( imout, &buf, 1);
+				}
+				stop_image_output( &imout );
+				free_scanline( &buf, True );
+				free( read_buf );
+			}   
+		}	  
+	}	 
+	if( im == NULL )
+		show_error( "invalid or unsupported TGA format in image file \"%s\"", path );
+
+	fclose( infile );
+	SHOW_TIME("image loading",started);
+	return im ;
+}
+/*************************************************************************/
+/* ARGB 																 */
+/*************************************************************************/
+ASImage *
+convert_argb2ASImage( ASVisual *asv, int width, int height, ARGB32 *argb, CARD8 *gamma_table )
+{
+	ASImage *im = NULL ;
+	ASImageOutput  *imout ;
+	im = create_asimage( width, height, 100 );
+	if((imout = start_image_output( NULL, im, ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT)) == NULL )
+	{
+   		destroy_asimage( &im );
+		return NULL;
+	}else
+	{	
+		ASScanline    buf;
+		int y ;
+		prepare_scanline( im->width, 0, &buf, True );
+		for( y = 0 ; y < height ; ++y ) 
+		{	  
+			int x ;
+			for( x = 0 ; x < width ; ++x ) 
+			{
+				ARGB32 c = argb[x];
+				buf.alpha[x] 	= ARGB32_ALPHA8(c);	
+				buf.red[x] 	= ARGB32_RED8(c);	  
+				buf.green[x] 	= ARGB32_GREEN8(c);	  
+				buf.blue[x] 	= ARGB32_BLUE8(c);	  
+			}	 
+			argb += width ;			
+			set_flags( buf.flags, SCL_DO_RED|SCL_DO_GREEN|SCL_DO_BLUE|SCL_DO_ALPHA );
+			imout->output_image_scanline( imout, &buf, 1);
+		}
+		stop_image_output( &imout );
+		free_scanline( &buf, True );
+	}   
+						
+	return im ;	
+}
+
+
+ASImage *
+argb2ASImage( const char *path, ASImageImportParams *params )
+{
+	ASVisual fake_asv ;
+	long argb_data_len = -1; 
+	char *argb_data = NULL ;
+	ASImage *im = NULL ;
+
+	memset( &fake_asv, 0x00, sizeof(ASVisual) );
+
+	argb_data = load_binary_file(path, &argb_data_len);
+	if(argb_data == NULL || argb_data_len < 8 )
+		show_error( "unable to load file \"%s\" file is either too big or is not readable.\n", path );
+	else
+	{
+		int width = ((CARD32*)argb_data)[0] ;
+		int height = ((CARD32*)argb_data)[1] ;
+		if( 2 + width*height > argb_data_len/sizeof(CARD32))
+		{
+			show_error( "file \"%s\" is too small for specified image size of %dx%d.\n", path, width, height );
+		}else
+			im = convert_argb2ASImage( &fake_asv, width, height, (ARGB32*)argb_data+2, params->gamma_table );
+	}
+	if( argb_data ) 
+		free( argb_data );
+	
+	return im ;
+}
+
