@@ -145,6 +145,7 @@ extern "C" {
 #define ALIGN_HCENTER (PAD_LEFT|PAD_RIGHT)
 #define ALIGN_VCENTER (PAD_TOP|PAD_BOTTOM)
 #define ALIGN_CENTER  (ALIGN_VCENTER|ALIGN_HCENTER)
+#define ALIGN_MASK	  (PAD_MASK|RESIZE_MASK|FIT_LABEL_SIZE)
 
 #define DEFAULT_TBAR_HSPACING        5
 #define DEFAULT_TBAR_VSPACING        1
@@ -281,6 +282,11 @@ typedef struct
 }
 ASGeometry;
 
+typedef struct
+{
+  unsigned int width, height;
+}ASSize;
+
 typedef struct ASRectangle
 {/* 64 bit safe : */
   long x, y;
