@@ -40,7 +40,7 @@
 #define LOCAL_DEBUG
 #include "../../configure.h"
 
-#include "asinternals.h"
+#include "asinternals.h" 
 
 #include <unistd.h>
 
@@ -49,6 +49,8 @@
 #include "../../libAfterStep/session.h"
 #include "../../libAfterStep/mystyle_property.h"
 #include "../../libAfterStep/wmprops.h"
+#include "../../libAfterStep/desktop_category.h"
+#include "../../libAfterStep/kde.h"
 
 #include "../../libAfterConf/afterconf.h"
 
@@ -174,22 +176,22 @@ struct config main_config[] = {
     {"Xzap", SetInts, (char **)&TmpFeel.Xzap, (int *)&dummy},
     {"Yzap", SetInts, (char **)&TmpFeel.Yzap, (int *)&dummy},
     {"AutoReverse", SetInts, (char **)&TmpFeel.AutoReverse, (int *)&dummy},
-    {"AutoTabThroughDesks", SetFlag, (char **)AutoTabThroughDesks, NULL},
+    {"AutoTabThroughDesks", SetFlag2, (char **)AutoTabThroughDesks, NULL},
     {"MWMFunctionHints", obsolete, (char **)0, NULL},
     {"MWMDecorHints", obsolete, (char **)0, NULL},
     {"MWMHintOverride", obsolete, (char **)0, NULL},
-    {"FollowTitleChanges", SetFlag, (char **)FollowTitleChanges, (int *)0},
-    {"PersistentMenus", SetFlag, (char **)PersistentMenus, (int *)0},
+    {"FollowTitleChanges", SetFlag2, (char **)FollowTitleChanges, (int *)0},
+    {"PersistentMenus", SetFlag2, (char **)PersistentMenus, (int *)0},
     {"NoSnapKey", SetModifier, (char **)&(TmpFeel.no_snaping_mod), (int *)0},
     {"ScreenEdgeAttraction", SetInts, (char **)&TmpFeel.EdgeAttractionScreen, &dummy},
     {"WindowEdgeAttraction", SetInts, (char **)&TmpFeel.EdgeAttractionWindow, &dummy},
-    {"DontRestoreFocus", SetFlag, (char **)DontRestoreFocus, &dummy},
+    {"DontRestoreFocus", SetFlag2, (char **)DontRestoreFocus, NULL},
     {"WindowBox", windowbox_parse, (char**)NULL, (int*)NULL},
     {"DefaultWindowBox", assign_string, (char**)&(TmpFeel.default_window_box_name), (int*)0},
     {"RecentSubmenuItems", SetInts, (char**)&TmpFeel.recent_submenu_items, (int*)&dummy},
 	{"WinListSortOrder", SetInts, (char**)&TmpFeel.winlist_sort_order, (int *)&dummy},
-	{"WinListHideIcons", SetFlag, (char**)WinListHideIcons, (int *)&dummy},
-	{"SuppressIcons", SetFlag, (char**)SuppressIcons, (int *)&dummy},
+	{"WinListHideIcons", SetFlag2, (char**)WinListHideIcons, NULL},
+	{"SuppressIcons", SetFlag2, (char**)SuppressIcons, NULL},
 
     /* look options */
 	/* obsolete stuff */
@@ -262,6 +264,7 @@ struct config main_config[] = {
 	{"CustomCursor"						, SetCustomCursor, (char **)0, (int *)0},
 	{"Cursor"							, SetCursor, (char **)0, (int *)0},
     /***********************************/
+    {"IconsGrowVertically", SetLookFlag, (char **)IconsGrowVertically, (int *) 0},
 	{"MenuPinOn"						, assign_string, &MenuPinOn, (int *)0},    /* menu pin */
     {"MArrowPixmap"						, assign_pixmap, (char **)&TmpLook.MenuArrow, (int *)0},   /* menu arrow */
     {"TitlebarNoPush"					, SetLookFlag, (char **)TitlebarNoPush, NULL},
@@ -300,12 +303,14 @@ struct config main_config[] = {
     {"MenuHiliteCompositionMethod"		, SetInts, (char **)&TmpLook.menu_hcm, &dummy},
     {"MenuStippleCompositionMethod"		, SetInts, (char **)&TmpLook.menu_scm, &dummy},
     {"ShadeAnimationSteps"				, SetInts, (char **)&TmpFeel.ShadeAnimationSteps, (int *)&dummy},
-    {"TitleButtonBalloonBorderHilite"	, bevel_parse, (char**)"afterstep", (int*)&(BalloonConfig.border_hilite)},
-    {"TitleButtonBalloonXOffset"		, SetInts, (char**)&(BalloonConfig.x_offset), NULL},
-    {"TitleButtonBalloonYOffset"		, SetInts, (char**)&(BalloonConfig.y_offset), NULL},
-    {"TitleButtonBalloonDelay"			, SetInts, (char**)&(BalloonConfig.delay), NULL},
-    {"TitleButtonBalloonCloseDelay"		, SetInts, (char**)&(BalloonConfig.close_delay), NULL},
-    {"TitleButtonBalloonStyle"			, assign_string, &(BalloonConfig.style), NULL},
+    {"TitleButtonBalloonBorderHilite"	, bevel_parse, (char**)"afterstep", (int*)&(BalloonConfig.BorderHilite)},
+    {"TitleButtonBalloonXOffset"		, SetInts, (char**)&(BalloonConfig.XOffset), NULL},
+    {"TitleButtonBalloonYOffset"		, SetInts, (char**)&(BalloonConfig.YOffset), NULL},
+    {"TitleButtonBalloonDelay"			, SetInts, (char**)&(BalloonConfig.Delay), NULL},
+    {"TitleButtonBalloonCloseDelay"		, SetInts, (char**)&(BalloonConfig.CloseDelay), NULL},
+    {"TitleButtonBalloonStyle"			, assign_string, &(BalloonConfig.Style), NULL},
+    {"TitleButtonBalloonTextPaddingX"	, SetInts, (char**)&(BalloonConfig.TextPaddingX), NULL},
+    {"TitleButtonBalloonTextPaddingY"	, SetInts, (char**)&(BalloonConfig.TextPaddingY), NULL},
     {"TitleButtonBalloons"				, SetFlag2, (char**)BALLOON_USED, (int*)&(BalloonConfig.set_flags)},
     {"TitleButton"						, SetTitleButton, (char **)1, (int *)0},
     {"KillBackgroundThreshold"			, SetInts, (char**)&(TmpLook.KillBackgroundThreshold), NULL },
@@ -613,11 +618,13 @@ merge_old_look_variables (MyLook *look)
 void
 InitFeel (ASFeel *feel, Bool free_resources)
 {
-    if (free_resources && feel)
-		destroy_asfeel( feel, True );
-    feel->magic = MAGIC_ASFEEL ;
-	init_asfeel( feel );
-
+	if( feel ) 
+	{	
+    	if (free_resources)
+			destroy_asfeel( feel, True );
+    	feel->magic = MAGIC_ASFEEL ;
+		init_asfeel( feel );
+	}
 }
 
 void 
@@ -682,8 +689,8 @@ InitLook (MyLook *look, Bool free_resources)
         DestroyMyFrameDefinitions (&MyFrameList);
 		if( LegacyFrameDef )
 			DestroyMyFrameDefinitions( &LegacyFrameDef );
-        if( BalloonConfig.style )
-            free( BalloonConfig.style );
+        if( BalloonConfig.Style )
+            free( BalloonConfig.Style );
 		for( i = 0  ; i < BACK_STYLES ; ++i )
 			if( MSWindowName[i] )
 				free( MSWindowName[i] );
@@ -820,7 +827,7 @@ MyFrame *add_myframe_from_def( ASHashTable *list, MyFrameDefinition *fd, ASFlagT
     for( i = 0 ; i < FRAME_PARTS ; ++i )
     {
         if( fd->parts[i] )
-            set_string_value(&(frame->part_filenames[i]), mystrdup(fd->parts[i]), NULL, 0);
+            set_string(&(frame->part_filenames[i]), mystrdup(fd->parts[i]));
         if( get_flags( fd->set_part_size, 0x01<<i ) )
         {
             frame->part_width[i] = max(fd->part_width[i],1);
@@ -850,9 +857,9 @@ MyFrame *add_myframe_from_def( ASHashTable *list, MyFrameDefinition *fd, ASFlagT
     for( i = 0 ; i < BACK_STYLES ; ++i )
     {
         if( fd->title_styles[i] )
-            set_string_value(&(frame->title_style_names[i]), mystrdup(fd->title_styles[i]), NULL, 0 );
+            set_string(&(frame->title_style_names[i]), mystrdup(fd->title_styles[i]) );
         if( fd->frame_styles[i] )
-            set_string_value(&(frame->frame_style_names[i]), mystrdup(fd->frame_styles[i]), NULL, 0 );
+            set_string(&(frame->frame_style_names[i]), mystrdup(fd->frame_styles[i]) );
     }
     if( get_flags( fd->set_title_attr, MYFRAME_TitleFBevelSet ) )
         frame->title_fbevel = fd->title_fbevel;
@@ -910,7 +917,7 @@ MyFrame *add_myframe_from_def( ASHashTable *list, MyFrameDefinition *fd, ASFlagT
 
     	if( fd->title_backs[i] )
     	{
-        	set_string_value(&(frame->title_back_filenames[i]), mystrdup(fd->title_backs[i]), NULL, 0 );
+        	set_string(&(frame->title_back_filenames[i]), mystrdup(fd->title_backs[i]) );
         	if( !get_flags( fd->set_title_attr, MYFRAME_TitleBackAlignSet_Start<<i ) )
         	{
             	frame->title_backs_align[i] = FIT_LABEL_WIDTH ;
@@ -1165,21 +1172,18 @@ FixLook( MyLook *look )
     LOCAL_DEBUG_OUT( "syncing %s","");
     ASSync(False);
 #endif
-    /* updating balloons look */
-    if( BalloonConfig.style == NULL )
-        BalloonConfig.style = mystrdup( "TitleButtonBalloon" );
 
 #if defined(LOCAL_DEBUG) && !defined(NO_DEBUG_OUTPUT)
     Print_balloonConfig ( &BalloonConfig );
 #endif
-    balloon_config2look( look, &BalloonConfig );
+    balloon_config2look( look, &BalloonConfig, "TitleButtonBalloon" );
     set_balloon_look( look->balloon_look );
 
     /* checking sanity of the move-resize window geometry :*/
     if( (look->resize_move_geometry.flags&(HeightValue|WidthValue)) != (HeightValue|WidthValue))
 	{
-		int width = 0;
-		int height = 0 ;
+		unsigned int width = 0;
+		unsigned int height = 0 ;
 		get_text_size( " +88888 x +88888 ", look->MSWindow[BACK_FOCUSED]->font.as_font, look->MSWindow[BACK_FOCUSED]->text_style, &width, &height );
     	if( !get_flags(look->resize_move_geometry.flags, WidthValue ) )
 	        look->resize_move_geometry.width = width + SIZE_VINDENT * 2;
@@ -1195,6 +1199,7 @@ FixLook( MyLook *look )
         enable_hints_support( look->supported_hints, HINTS_ICCCM );
         enable_hints_support( look->supported_hints, HINTS_Motif );
         enable_hints_support( look->supported_hints, HINTS_Gnome );
+        enable_hints_support( look->supported_hints, HINTS_KDE );
         enable_hints_support( look->supported_hints, HINTS_ExtendedWM );
         enable_hints_support( look->supported_hints, HINTS_ASDatabase );
         enable_hints_support( look->supported_hints, HINTS_GroupLead );
@@ -1281,6 +1286,10 @@ redecorate_aswindow_iter_func(void *data, void *aux_data)
     ASWindow *asw = (ASWindow*)data;
     if(asw )
 	{
+		/* need to invalidate all MyStyles at this point ??? */
+		
+		invalidate_window_mystyles( asw );
+		
 		redecorate_window( asw, False );
         if( asw->internal && asw->internal->on_look_feel_changed )
             asw->internal->on_look_feel_changed( asw->internal, &Scr.Feel, &Scr.Look, ASFLAGS_EVERYTHING );
@@ -1311,9 +1320,23 @@ advertise_tbar_props()
 		{ &_AS_BUTTON_SHADE, 	&_AS_BUTTON_SHADE_PRESSED, 		F_SHADE, 	NULL },
 		{ &_AS_BUTTON_MENU, 	&_AS_BUTTON_MENU_PRESSED, 		F_POPUP, 	NULL }};
 
-	
-	props.align = frame->title_align ;
-	props.bevel = frame->title_fbevel|frame->title_ubevel ;
+	memset( &props, 0x00, sizeof(props));
+	if( get_flags( frame->set_title_attr, MYFRAME_TitleAlignSet ) )
+	{
+		props.align = frame->title_align ;
+#if (NO_ALIGN==0)		
+		if( props.align == 0 )  		   
+			props.align = ~ALIGN_MASK ;
+#endif
+	}
+	if( get_flags( frame->set_title_attr, MYFRAME_TitleFBevelSet|MYFRAME_TitleUBevelSet ) )
+	{	
+		props.bevel = frame->title_fbevel|frame->title_ubevel ;
+#if (NO_HILITE==0)
+		if( props.bevel == 0 )
+			props.bevel = ~HILITE_MASK;
+#endif			
+	}
 	props.title_h_spacing = frame->title_h_spacing ;
 	props.title_v_spacing = frame->title_v_spacing ;
 	props.buttons_h_border = max(Scr.Look.TitleButtonXOffset[0],Scr.Look.TitleButtonXOffset[1]);
@@ -1382,30 +1405,6 @@ advertise_tbar_props()
 /*************************************************************************/
 /* reading confiug files now :                                           */
 /*************************************************************************/
-void
-ParseDatabase (const char *file)
-{
-    struct name_list *list = NULL ;
-
-	/* memory management for parsing buffer */
-	if (file == NULL)
-        return ;
-
-    list = ParseDatabaseOptions (file, "afterstep");
-    if( list )
-    {
-        Database = build_asdb( list );
-        if( is_output_level_under_threshold( OUTPUT_LEVEL_DATABASE ) )
-            print_asdb( NULL, NULL, Database );
-        while (list != NULL)
-            delete_name_list (&(list));
-    }else
-        show_progress( "no database records loaded." );
-    /* XResources : */
-    load_user_database();
-}
-
-
 int
 ParseConfigFile (const char *file, char **tline)
 {
@@ -1458,7 +1457,17 @@ LoadASConfig (int thisdesktop, ASFlagType what)
     char            *tline = NULL;
     ASImageManager  *old_image_manager = NULL ;
     ASFontManager   *old_font_manager  = NULL ;
-    
+	FunctionData gtkrc_signal_func ;
+	FunctionData kde_signal_func ;
+	ASEvent dummy_event = {0};
+	
+	init_func_data( &gtkrc_signal_func );
+    gtkrc_signal_func.func = F_SIGNAL_RELOAD_GTK_RCFILE ;
+	init_func_data( &kde_signal_func );
+    kde_signal_func.func = F_KIPC_SEND_MESSAGE_ALL ;
+	kde_signal_func.func_val[0] = KIPC_PaletteChanged ; 
+	kde_signal_func.func_val[1] = 0 ; 
+
     cover_desktop();
 
 #ifndef DIFFERENTLOOKNFEELFOREACHDESKTOP
@@ -1474,7 +1483,7 @@ LoadASConfig (int thisdesktop, ASFlagType what)
         const char *const_configfile;
         if (get_flags(what, PARSE_BASE_CONFIG))
 		{
-			if( ReloadASEnvironment( &old_image_manager, &old_font_manager, NULL, get_flags(what, PARSE_LOOK_CONFIG) ) )
+			if( ReloadASEnvironment( &old_image_manager, &old_font_manager, NULL, get_flags(what, PARSE_LOOK_CONFIG), True ) )
 			{
 				if( !get_flags(what, PARSE_LOOK_CONFIG) )
 				{
@@ -1493,7 +1502,6 @@ LoadASConfig (int thisdesktop, ASFlagType what)
 
         if (get_flags(what, PARSE_LOOK_CONFIG))
 		{
-
 			stop_all_background_xfer();
 			LoadColorScheme();
 
@@ -1531,7 +1539,11 @@ LoadASConfig (int thisdesktop, ASFlagType what)
                 display_progress( True, "LOOK configuration file cannot be found!");
                 clear_flags(what, PARSE_LOOK_CONFIG);
             }
-        }
+        	if( UpdateGtkRC(Environment) ) 
+				ExecuteFunction (&gtkrc_signal_func, &dummy_event, -1);
+			if( UpdateKCSRC() ) 
+				ExecuteFunction (&kde_signal_func, &dummy_event, -1);
+		}
         if (get_flags(what, PARSE_FEEL_CONFIG))
 		{
             if( (const_configfile = get_session_file (Session, thisdesktop, F_CHANGE_FEEL, False) ) != NULL )
@@ -1542,7 +1554,12 @@ LoadASConfig (int thisdesktop, ASFlagType what)
                 InitFeel (&Scr.Feel, True);
                 if (tline == NULL)
                     tline = safemalloc (MAXLINELENGTH + 1);
-                display_progress( True, "Parsing menu entries and checking availability ...");
+				
+				display_progress( True, "Reloading and merging desktop categories ...");
+				ReloadCategories(False);
+				UpdateCategoriesCache();
+                
+				display_progress( True, "Parsing menu entries and checking availability ...");
                 MeltStartMenu (tline);
                 display_progress( False, "Done..");
                 ParseConfigFile (const_configfile, &tline);
@@ -1595,24 +1612,21 @@ LoadASConfig (int thisdesktop, ASFlagType what)
         }
 
         if (get_flags(what, PARSE_DATABASE_CONFIG))
-        {
-            if( (configfile = make_session_file(Session, DATABASE_FILE, False )) != NULL )
-            {
-                InitDatabase (True);
-                ParseDatabase (configfile);
-                show_progress("DATABASE configuration loaded from \"%s\" ...", configfile);
-                display_progress( True, "DATABASE configuration loaded from \"%s\" .", configfile);
-                free( configfile );
-            }else
-            {
-                show_warning("DATABASE configuration file cannot be found!");
-                display_progress( True, "DATABASE configuration file cannot be found!");
+		{
+			if( !ReloadASDatabase() )
+			{
+		        display_progress( True, "DATABASE configuration file cannot be found!");
                 clear_flags(what, PARSE_DATABASE_CONFIG);
-            }
-        }
+			}else
+			{
+				configfile  = make_session_file(Session, DATABASE_FILE, False );
+	    	    display_progress( True, "DATABASE configuration loaded from \"%s\" .", configfile);
+				free( configfile );
+			}
+		}
 	} else
 	{
-		ReloadASEnvironment( &old_image_manager, &old_font_manager, NULL, True );
+		ReloadASEnvironment( &old_image_manager, &old_font_manager, NULL, True, True );
 		LoadColorScheme();
 		memset( &TmpLook, 0x00, sizeof(TmpLook));
 		InitLook (&TmpLook, False );
@@ -1620,13 +1634,16 @@ LoadASConfig (int thisdesktop, ASFlagType what)
 		memset( &TmpFeel, 0x00, sizeof(TmpFeel));
 		InitFeel (&TmpFeel, False );
         InitFeel (&Scr.Feel, True);
-        InitDatabase (True);
         ParseConfigFile (Session->overriding_file, &tline);
+        ReloadASDatabase();
 		merge_look( &Scr.Look, &TmpLook );
 		merge_feel( &Scr.Feel, &TmpFeel );
         show_progress("AfterStep configuration loaded from \"%s\" ...", Session->overriding_file);
         display_progress( True, "AfterStep configuration loaded from \"%s\".", Session->overriding_file);
         what = PARSE_EVERYTHING ;
+		if( UpdateGtkRC(Environment) ) 
+			ExecuteFunction (&gtkrc_signal_func, &dummy_event, -1);
+		UpdateKCSRC();
     }
 
 	/* let's free the memory used for parsing */
@@ -1887,7 +1904,8 @@ SetCustomCursor (char *text, FILE * fd, char **arg, int *junk)
 	int           num, cursor_num;
 	char          f_cursor[1024], f_mask[1024];
 	Pixmap        cursor = None, mask = None;
-	int           width, height, x, y;
+	unsigned int  width, height;
+	int x, y;
 	XColor        fore, back;
 	char         *path;
     Cursor new_c ;
