@@ -60,7 +60,7 @@
  *  DispatchEvent - calls appropriate handler for the event
  ************************************************************************/
 void DigestEvent    ( ASEvent *event );
-void afterstep_wait_pipes_input ();
+void afterstep_wait_pipes_input (int timeout_sec);
 
 void
 HandleEvents ()
@@ -88,7 +88,7 @@ HandleEvents ()
             }
             ExecutePendingFunctions();
         }
-        afterstep_wait_pipes_input ();
+        afterstep_wait_pipes_input (0);
         ExecutePendingFunctions();
     }
 }
@@ -121,7 +121,7 @@ LOCAL_DEBUG_OUT( "checking masked events ...%s", "" );
             if( Scr.moveresize_in_progress == NULL )
                 return;
         }
-        afterstep_wait_pipes_input ();
+        afterstep_wait_pipes_input (0);
     }
 }
 
@@ -293,7 +293,7 @@ WaitWindowLoop( char *pattern, long timeout )
 
 		if( time(NULL) > end_time )
 			break;
-		afterstep_wait_pipes_input ();
+		afterstep_wait_pipes_input (1);
 	}
 	destroy_ashint_window( &hint );
     return NULL;
@@ -362,8 +362,15 @@ DigestEvent( ASEvent *event )
 			event->client = NULL ;
 		}else
 		{
-			event->widget = NULL ;	
-			event->client = window2ASWindow( event->w );
+			if( (event->eclass & ASE_POINTER_EVENTS) != 0 && is_balloon_click( &(event->x) ) )
+			{	
+				event->client = NULL ;
+				event->widget = NULL ;	 
+			}else
+			{	
+				event->widget = NULL ;	
+				event->client = window2ASWindow( event->w );
+			}
 		}	 
 	}
 
@@ -695,8 +702,10 @@ DispatchEvent ( ASEvent *event, Bool deffered )
 void
 HandleFocusIn ( ASEvent *event )
 {
-    while (ASCheckTypedEvent (FocusIn, &event->x));
-    DigestEvent( event );
+    int events_count = 0 ;
+	while (ASCheckTypedEvent (FocusIn, &event->x)) events_count++;
+	if( events_count > 0 ) 
+    	DigestEvent( event );
 
     if( get_flags( AfterStepState, ASS_WarpingMode ) )
         ChangeWarpingFocus( event->client );
@@ -800,6 +809,7 @@ HandlePropertyNotify (ASEvent *event)
     ASWindow       *asw;
     XPropertyEvent *xprop = &(event->x.xproperty);
     Atom atom = xprop->atom ;
+	XEvent prop_xev ; 
 
 	/* force updates for "transparent" windows */
     if (atom == _XROOTPMAP_ID && event->w == Scr.Root)
@@ -833,10 +843,15 @@ HandlePropertyNotify (ASEvent *event)
 					move_menu (menu, (*menu).x, (*menu).y);
         }
 #endif
+		return;
     }
 
-    if( (asw = event->client) == NULL )
+    if( (asw = event->client) == NULL || ASWIN_GET_FLAGS( asw, AS_Dead ))
+	{
+		if( event->w != Scr.Root )
+			while (XCheckTypedWindowEvent (dpy, event->w, PropertyNotify, &prop_xev));
         return ;
+	}
     else
     {
 		char *prop_name = NULL;
@@ -847,12 +862,24 @@ HandlePropertyNotify (ASEvent *event)
     if( IsNameProp(atom))
     {
 		char *old_name = get_flags( asw->internal_flags, ASWF_NameChanged )?NULL:mystrdup( ASWIN_NAME(asw) );
+		
+		/* we want to check if there were some more events generated 
+		 * as window names tend to change multiple properties : */
+		while (XCheckTypedWindowEvent (dpy, asw->w, PropertyNotify, &prop_xev))
+			if( !IsNameProp(prop_xev.xproperty.atom) ) 
+			{	
+				XPutBackEvent( dpy, &prop_xev );
+				break;
+			}
+		
 		/*ASFlagType old_hflags = asw->hints->flags ; */
 		show_debug( __FILE__, __FUNCTION__, __LINE__, "name prop changed..." );
-        if( update_property_hints_manager( asw->w, xprop->atom,
-                                        Scr.Look.supported_hints,
-										Database,
-                                        asw->hints, asw->status ) )
+        if( get_flags( Scr.Feel.flags, FollowTitleChanges) )
+    	    on_window_hints_changed( asw );
+    	else if( update_property_hints_manager( asw->w, xprop->atom,
+             	                           		Scr.Look.supported_hints,
+												Database,
+                                        		asw->hints, asw->status ) )
         {
             if( ASWIN_GET_FLAGS( asw, AS_Dead ) )
 				return;
@@ -862,15 +889,11 @@ HandlePropertyNotify (ASEvent *event)
 			if( old_name && strcmp( old_name, ASWIN_NAME(asw) ) != 0 )
 				set_flags( asw->internal_flags, ASWF_NameChanged );
             /* fix the name in the title bar */
-            if( get_flags( Scr.Feel.flags, FollowTitleChanges) )
-	    	    on_window_hints_changed( asw );
-	    	else
-	    	{
-				if (!ASWIN_GET_FLAGS(asw, AS_Iconic))
-           	    	on_window_title_changed( asw, True );
-	        	broadcast_window_name( asw );
-        		broadcast_icon_name( asw );
-		    }
+			if (!ASWIN_GET_FLAGS(asw, AS_Iconic))
+       	    	on_window_title_changed( asw, True );
+			broadcast_res_names( asw );
+        	broadcast_window_name( asw );
+       		broadcast_icon_name( asw );
         }
 		if( old_name )
 			free( old_name );
@@ -1339,7 +1362,8 @@ HandleConfigureRequest ( ASEvent *event )
 		return;
 	}
 
-	if( ASWIN_HFLAGS(asw, AS_IgnoreConfigRequest ) )
+	if( ASWIN_HFLAGS(asw, AS_IgnoreConfigRequest ) ||
+		ASWIN_GET_FLAGS( asw, AS_Fullscreen ) )
 	{	
 		LOCAL_DEBUG_OUT( "Ignoring ConfigureRequest for client %p as required by hints", asw );
 		send_canvas_configure_notify(asw->frame_canvas, asw->client_canvas);
@@ -1464,7 +1488,7 @@ void HandleShmCompletion(ASEvent *event)
  *
  ****************************************************************************/
 void
-afterstep_wait_pipes_input()
+afterstep_wait_pipes_input(int timeout_sec)
 {
 	fd_set        in_fdset, out_fdset;
 	int           retval;
@@ -1507,6 +1531,12 @@ afterstep_wait_pipes_input()
 	/* watch for timeouts */
 	if (timer_delay_till_next_alarm ((time_t *) & tv.tv_sec, (time_t *) & tv.tv_usec))
 		t = &tv;
+	else if( timeout_sec > 0 )
+	{
+		t = &tv ; 
+		tv.tv_sec = timeout_sec ; 
+		tv.tv_usec = 0 ;
+	}
     retval = PORTABLE_SELECT(min (max_fd + 1, fd_width),&in_fdset,&out_fdset,NULL,t);
 
 	if (retval > 0)
