@@ -40,10 +40,15 @@
 #include "../../libAfterStep/session.h"
 #include "../../libAfterStep/moveresize.h"
 #include "../../libAfterStep/mylook.h"
+#include "../../libAfterStep/desktop_category.h"
+#include "../../libAfterStep/wmprops.h"
+#include "../../libAfterConf/afterconf.h"
+#include "../../libAfterStep/kde.h"
 
 static as_function_handler function_handlers[F_FUNCTIONS_NUM] ;
 
 void ExecuteComplexFunction ( ASEvent *event, char *name );
+Bool desktop_category2complex_function( const char *name, const char *category_name );
 
 /* list of available handlers : */
 void beep_func_handler( FunctionData *data, ASEvent *event, int module );
@@ -65,7 +70,9 @@ void close_func_handler( FunctionData *data, ASEvent *event, int module );
 void restart_func_handler( FunctionData *data, ASEvent *event, int module );
 void exec_func_handler( FunctionData *data, ASEvent *event, int module );
 void exec_in_term_func_handler( FunctionData *data, ASEvent *event, int module );
+void exec_tool_func_handler( FunctionData *data, ASEvent *event, int module );
 void change_background_func_handler( FunctionData *data, ASEvent *event, int module );
+void change_back_foreign_func_handler( FunctionData *data, ASEvent *event, int module );
 void change_config_func_handler( FunctionData *data, ASEvent *event, int module );
 void change_theme_func_handler( FunctionData *data, ASEvent *event, int module );
 void install_file_func_handler( FunctionData *data, ASEvent *event, int module );
@@ -79,15 +86,20 @@ void desk_func_handler( FunctionData *data, ASEvent *event, int module );
 void deskviewport_func_handler( FunctionData *data, ASEvent *event, int module );
 void module_func_handler( FunctionData *data, ASEvent *event, int module );
 void killmodule_func_handler( FunctionData *data, ASEvent *event, int module );
+void killallmodules_func_handler( FunctionData *data, ASEvent *event, int module );
+void restartmodule_func_handler( FunctionData *data, ASEvent *event, int module );
 void popup_func_handler( FunctionData *data, ASEvent *event, int module );
 void quit_func_handler( FunctionData *data, ASEvent *event, int module );
 void windowlist_func_handler( FunctionData *data, ASEvent *event, int module );
 void quickrestart_func_handler( FunctionData *data, ASEvent *event, int module );
 void send_window_list_func_handler( FunctionData *data, ASEvent *event, int module );
 void save_workspace_func_handler( FunctionData *data, ASEvent *event, int module );
+void signal_reload_GTKRC_file_handler( FunctionData *data, ASEvent *event, int module );
+void KIPC_send_message_all_handler( FunctionData *data, ASEvent *event, int module );
 void test_func_handler( FunctionData *data, ASEvent *event, int module );
 void screenshot_func_handler( FunctionData *data, ASEvent *event, int module );
 void swallow_window_func_handler( FunctionData *data, ASEvent *event, int module );
+void modulelist_func_handler( FunctionData *data, ASEvent *event, int module );
 
 
 /* handlers initialization function : */
@@ -115,7 +127,8 @@ void SetupFunctionHandlers()
 
     function_handlers[F_CHANGE_WINDOWS_DESK]= change_desk_func_handler;
 
-    function_handlers[F_MAXIMIZE]           =
+    function_handlers[F_MAXIMIZE]       =
+    function_handlers[F_FULLSCREEN]     =
 	function_handlers[F_SHADE]          =
 	function_handlers[F_STICK]          = toggle_status_func_handler;
 
@@ -133,16 +146,20 @@ void SetupFunctionHandlers()
     function_handlers[F_PIN_MENU]           = pin_menu_func_handler ;
     function_handlers[F_DESTROY] =
 	function_handlers[F_DELETE] =
-	function_handlers[F_CLOSE]          = close_func_handler ;
+	function_handlers[F_CLOSE]          	= close_func_handler ;
     function_handlers[F_RESTART]            = restart_func_handler ;
 
-    function_handlers[F_EXEC] =
-		function_handlers[F_Swallow] =
+    function_handlers[F_EXEC] 				=
+		function_handlers[F_Swallow] 		=
 		function_handlers[F_MaxSwallow] 	= exec_func_handler ;
 
 	function_handlers[F_ExecInTerm]       	= exec_in_term_func_handler ;
+	
+	function_handlers[F_ExecBrowser]       	= 
+		function_handlers[F_ExecEditor]     = exec_tool_func_handler ;
 
     function_handlers[F_CHANGE_BACKGROUND]  = change_background_func_handler;
+	function_handlers[F_CHANGE_BACKGROUND_FOREIGN]  = change_back_foreign_func_handler;
 
     function_handlers[F_CHANGE_LOOK] =
 	function_handlers[F_CHANGE_FEEL]    =
@@ -163,6 +180,8 @@ void SetupFunctionHandlers()
 
 
 	function_handlers[F_SAVE_WORKSPACE]     = save_workspace_func_handler;
+	function_handlers[F_SIGNAL_RELOAD_GTK_RCFILE] = signal_reload_GTKRC_file_handler;
+	function_handlers[F_KIPC_SEND_MESSAGE_ALL] = KIPC_send_message_all_handler;
 
     function_handlers[F_REFRESH]            = refresh_func_handler ;
 #ifndef NO_VIRTUAL
@@ -180,11 +199,16 @@ void SetupFunctionHandlers()
 	function_handlers[F_MaxSwallowModule]= module_func_handler ;
 
     function_handlers[F_KILLMODULEBYNAME]   = killmodule_func_handler ;
+	function_handlers[F_RESTARTMODULEBYNAME]= restartmodule_func_handler ;
+	function_handlers[F_KILLALLMODULESBYNAME] = killallmodules_func_handler ;
     function_handlers[F_POPUP]              = popup_func_handler ;
     function_handlers[F_QUIT]               = quit_func_handler ;
 #ifndef NO_WINDOWLIST
     function_handlers[F_WINDOWLIST]         = windowlist_func_handler ;
 #endif /* ! NO_WINDOWLIST */
+	function_handlers[F_STOPMODULELIST]     = 
+	function_handlers[F_RESTARTMODULELIST]  = modulelist_func_handler ;
+
     function_handlers[F_QUICKRESTART]       = quickrestart_func_handler ;
     function_handlers[F_SEND_WINDOW_LIST]   = send_window_list_func_handler ;
     function_handlers[F_Test]               = test_func_handler ;
@@ -421,24 +445,59 @@ DoExecuteFunction ( ASScheduledFunction *sf )
 
 	}
 
-    if( function_handlers[func] || func == F_FUNCTION )
+    if( function_handlers[func] || func == F_FUNCTION || func == F_CATEGORY)
     {
+		char *complex_func_name = COMPLEX_FUNCTION_NAME(data);
+
 		data->func = func ;
 		if( event->client )
-	    	if( !check_allowed_function2( data->func, event->client->hints) )
+		{
+			if( get_flags( event->client->status->flags, AS_Fullscreen ) &&
+				(data->func == F_MOVE || 
+				 data->func == F_RESIZE ||
+				 data->func == F_MAXIMIZE ) )
+			{
+				LOCAL_DEBUG_OUT( "function \"%s\" is not allowed for the Fullscreen window",COMPLEX_FUNCTION_NAME(data));
+				func = data->func = F_BEEP ;				                     
+			}else if( !check_allowed_function2( data->func, event->client->hints) )
 	    	{
 				LOCAL_DEBUG_OUT( "function \"%s\" is not allowed for the specifyed window (mask 0x%lX)", COMPLEX_FUNCTION_NAME(data), ASWIN_FUNC_MASK(event->client));
 				func = data->func = F_BEEP ;
 	    	}
+		}
 
 		if( get_flags( AfterStepState, ASS_WarpingMode ) &&
 	    	function_handlers[func] != warp_func_handler )
 	    	EndWarping();
 
+		if( func == F_CATEGORY )
+		{
+			char *cat_name = data->text?data->text:data->name ; 
+			if( cat_name == NULL ) 
+				func = F_BEEP ;
+			else
+			{	
+				complex_func_name = safemalloc( sizeof("CATEGORY()")+strlen(cat_name)+1);
+				sprintf( complex_func_name, "CATEGORY(%s)", cat_name );
+				if( get_complex_function( complex_func_name ) != NULL ) 
+				{	
+					func = F_FUNCTION ;
+				}else
+				{	
+					if( desktop_category2complex_function( complex_func_name, cat_name ) )
+						func = F_FUNCTION ; 
+					else
+						func = F_BEEP ;
+				}
+			}
+		}
+			 
 		if( func == F_FUNCTION )
-		    ExecuteComplexFunction (event, COMPLEX_FUNCTION_NAME(data));
+		    ExecuteComplexFunction (event, complex_func_name);
 		else
 		    function_handlers[func]( data, event, sf->module );
+		if( complex_func_name && complex_func_name != data->name && complex_func_name != data->text )
+			free( complex_func_name );
     }
     destroy_scheduled_function(sf);
 }
@@ -652,6 +711,52 @@ ExecuteBatch ( ComplexFunction *batch )
 }
 
 
+Bool desktop_category2complex_function( const char *name, const char *category_name )
+{
+	ASCategoryTree *ct = NULL ;  
+	ASDesktopCategory *dc = NULL ;
+    ComplexFunction *func ;
+	char **entries ; 
+	int i, entries_num ; 
+
+	if( (dc = name2desktop_category( category_name, &ct )) == NULL ) 
+		return False; 
+
+	entries_num = PVECTOR_USED(dc->entries);
+	if( entries_num == 0 ) 
+		return False;
+	entries = PVECTOR_HEAD(char*, dc->entries );
+    
+	if( (func = new_complex_func( Scr.Feel.ComplexFunctions, (char*)name)) == NULL ) 
+		return False;
+	
+	func->items_num = 0 ;
+	func->items = safecalloc(entries_num, sizeof(FunctionData));
+	for( i = 0 ; i < entries_num ; ++i ) 
+	{	
+		ASDesktopEntry *de ;
+		FunctionData *fdata = &(func->items[func->items_num]);
+		if( (de = fetch_desktop_entry( ct, entries[i] ))== NULL ) 
+			continue;
+		if( de->type != ASDE_TypeApplication )
+			continue;
+		
+		++(func->items_num);
+		init_func_data( fdata );
+		fdata->name = mystrdup("I");  /* for immidiate execution */ 
+		
+		if( get_flags( de->flags, ASDE_Terminal ) )
+			fdata->func = F_ExecInTerm;	   
+		else if( get_flags( de->flags, ASDE_ASModule ) )
+			fdata->func = F_MODULE;	   
+		else
+		 	fdata->func = F_EXEC;	
+        fdata->text = mystrdup( de->clean_exec );
+	}	
+
+	return True;	   
+}	 
+
 /***************************************************************************************
  *
  * All The Handlers Go Below, Go Below,
@@ -854,9 +959,9 @@ LOCAL_DEBUG_CALLER_OUT( "function %ld (val0 = %ld), event %d, window 0x%lX, wind
 	if (ASWIN_GET_FLAGS(event->client, AS_Iconic) )
 	{
 		if (data->func_val[0] <= 0)
-		set_window_wm_state( event->client, False );
+		set_window_wm_state( event->client, False, False );
 	}else if (data->func_val[0] >= 0)
-		set_window_wm_state( event->client, True );
+		set_window_wm_state( event->client, True, False );
 	}
 }
 
@@ -882,16 +987,26 @@ void raise_it_func_handler( FunctionData *data, ASEvent *event, int module )
 
 void setlayer_func_handler( FunctionData *data, ASEvent *event, int module )
 {
-    register int func = data->func ;
-    int layer = 0 ;
+    if( event->client )
+	{
+    	register int func = data->func ;
+	    int layer = 0 ;
 
-    if( func == F_PUTONTOP )
-	layer = AS_LayerTop ;
-    else if( func == F_PUTONBACK )
-	layer = AS_LayerBack ;
-    else
-	layer = (func == F_TOGGLELAYER && ASWIN_LAYER(event->client) == data->func_val[1])?data->func_val[1]: data->func_val[0] ;
-    change_aswindow_layer( event->client, layer );
+    	if( func == F_PUTONTOP )
+			layer = AS_LayerTop ;
+    	else if( func == F_PUTONBACK )
+			layer = AS_LayerBack ;
+    	else if( func == F_TOGGLELAYER )
+		{
+			layer = ASWIN_LAYER(event->client) ;
+			if( data->func_val[0] == 0 ) 
+				layer += data->func_val[1];
+			else
+				layer += data->func_val[0];
+		}else
+			layer = data->func_val[0] ;
+    	change_aswindow_layer( event->client, layer );
+	}
 }
 
 void change_desk_func_handler( FunctionData *data, ASEvent *event, int module )
@@ -904,7 +1019,7 @@ void toggle_status_func_handler( FunctionData *data, ASEvent *event, int module
 {
     ASFlagType toggle_flags = 0;
     if( data->func == F_STICK )
-	toggle_flags = AS_Sticky ;
+		toggle_flags = AS_Sticky ;
     else if( data->func == F_MAXIMIZE )
 	{
 		if( data->func_val[0] > 0 || data->func_val[1] == 0  )
@@ -917,9 +1032,11 @@ void toggle_status_func_handler( FunctionData *data, ASEvent *event, int module
 			event->client->maximize_ratio_y = (data->unit_val[1] == 0 )? data->func_val[1]: (data->func_val[1]*data->unit_val[1]*100)/Scr.MyDisplayHeight ;
 		}
     }else if( data->func == F_SHADE )
-	toggle_flags = AS_Shaded ;
-    else
-	return ;
+		toggle_flags = AS_Shaded ;
+    else if( data->func == F_FULLSCREEN )
+		toggle_flags = AS_Fullscreen ;
+	else
+		return ;
     toggle_aswindow_status( event->client, toggle_flags );
 }
 
@@ -1119,12 +1236,12 @@ char *parse_term_cmdl( const char *term_name, const char *term_command, const ch
 
 void exec_in_term_func_handler( FunctionData *data, ASEvent *event, int module )
 {
-	if( Environment->term_command != NULL && data->text != NULL ) 
+	if( Environment->tool_command[ASTool_Term] != NULL && data->text != NULL ) 
 	{
 		char *full_cmdl = NULL;
-		char *term_name = strrchr( Environment->term_command, '/' );
-		term_name = (term_name == NULL )? Environment->term_command: term_name+1 ;
-		full_cmdl = parse_term_cmdl( term_name, Environment->term_command, data->text );
+		char *term_name = strrchr( Environment->tool_command[ASTool_Term], '/' );
+		term_name = (term_name == NULL )? Environment->tool_command[ASTool_Term]: term_name+1 ;
+		full_cmdl = parse_term_cmdl( term_name, Environment->tool_command[ASTool_Term], data->text );
 		if( full_cmdl ) 
 		{	
 			LOCAL_DEBUG_OUT( "full_cmdl = [%s]", full_cmdl );
@@ -1140,6 +1257,24 @@ void exec_in_term_func_handler( FunctionData *data, ASEvent *event, int module )
 	}
 }
 
+void exec_tool_func_handler( FunctionData *data, ASEvent *event, int module )
+{
+	ASToolType tool = (data->func == F_ExecBrowser)?ASTool_Browser:ASTool_Editor ;
+	if( Environment->tool_command[tool] != NULL && data->text != NULL ) 
+	{
+		char *full_cmdl = safemalloc( strlen(Environment->tool_command[tool]) +1+strlen(data->text)+1 );
+		sprintf( full_cmdl, "%s %s", Environment->tool_command[tool], data->text );
+		LOCAL_DEBUG_OUT( "full_cmdl = [%s]", full_cmdl );
+   		XGrabPointer(  dpy, Scr.Root, True,
+			      		ButtonPressMask | ButtonReleaseMask,
+				  		GrabModeAsync, GrabModeAsync, Scr.Root, Scr.Feel.cursors[ASCUR_Wait], CurrentTime);
+    	XSync (dpy, 0);
+    	spawn_child( full_cmdl, -1, -1, None, C_NO_CONTEXT, True, False, NULL );
+    	XUngrabPointer (dpy, CurrentTime);
+		free( full_cmdl );
+    	XSync (dpy, 0);
+	}
+}
 
 static int _as_config_change_recursion = 0 ;
 static int _as_config_change_count = 0 ;
@@ -1175,9 +1310,10 @@ commit_config_change( int func )
     }
 }
 
-void change_background_func_handler( FunctionData *data, ASEvent *event, int module )
+static void change_background_internal( FunctionData *data, Bool autoscale )
 {
     char tmpfile[256], *realfilename ;
+	Bool success = False ;
 
 	++_as_config_change_recursion;
     XGrabPointer (dpy, Scr.Root, True, ButtonPressMask | ButtonReleaseMask,
@@ -1185,20 +1321,79 @@ void change_background_func_handler( FunctionData *data, ASEvent *event, int mod
     XSync (dpy, 0);
 
     if (Scr.screen == 0)
-	sprintf (tmpfile, BACK_FILE, Scr.CurrentDesk);
+		sprintf (tmpfile, BACK_FILE, Scr.CurrentDesk);
     else
-	sprintf (tmpfile, BACK_FILE ".scr%ld", Scr.CurrentDesk, Scr.screen);
+		sprintf (tmpfile, BACK_FILE ".scr%ld", Scr.CurrentDesk, Scr.screen);
 
     realfilename = make_session_data_file(Session, False, 0, tmpfile, NULL );
     cover_desktop();
-    display_progress( True, "Copying selected background \"%s\" into \"%s\" ...", data->text, realfilename);
-    LOCAL_DEBUG_OUT( "Copying selected background \"%s\" into \"%s\" ...", data->text, realfilename);
-    if (CopyFile (data->text, realfilename) == 0)
+	if( autoscale ) 
+	{
+		ASImage *src_im = get_asimage( Scr.image_manager, data->text, 0xFFFFFFFF, 0 );
+		if( src_im ) 
+		{
+			if( src_im->width >= 600 && src_im->height >= 600 ) 
+			{
+				int clip_width = src_im->width ; 
+				int clip_height = src_im->height ; 
+				ASImage *tiled = src_im ;
+
+				if( clip_width*Scr.MyDisplayHeight > ((clip_height*5)/4)*Scr.MyDisplayWidth ) 
+					clip_width = (((clip_height*5)/4)*Scr.MyDisplayWidth)/Scr.MyDisplayHeight ;	
+				else if( ((clip_width*5)/4)*Scr.MyDisplayHeight < clip_height*Scr.MyDisplayWidth ) 
+				 	clip_height = (((clip_width*5)/4)*Scr.MyDisplayHeight)/Scr.MyDisplayWidth ;
+				if( clip_width != src_im->width || clip_height != src_im->height ) 
+				{
+					display_progress( True, "Croping background to %dx%d ...", clip_width, clip_height);
+   					LOCAL_DEBUG_OUT( "Croping background to %dx%d ...", clip_width, clip_height);
+					tiled = tile_asimage( Scr.asv, src_im, 
+											(src_im->width-clip_width)/2, 
+											(src_im->height-clip_height)/2, 
+											clip_width, clip_height, TINT_LEAVE_SAME, 
+											ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT );
+				}
+				if( tiled ) 
+				{
+					ASImage *scaled = tiled ; 
+					if( tiled->width != Scr.MyDisplayWidth || tiled->height != Scr.MyDisplayHeight ) 
+					{
+						display_progress( True, "Scaling background to %dx%d ...", Scr.MyDisplayWidth, Scr.MyDisplayHeight);
+    					LOCAL_DEBUG_OUT( "Scaling background to %dx%d ...", Scr.MyDisplayWidth, Scr.MyDisplayHeight);
+						scaled = scale_asimage( Scr.asv, tiled, 
+												Scr.MyDisplayWidth, 
+												Scr.MyDisplayHeight, 
+												ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT );
+					}
+					if( scaled && scaled != src_im )
+					{
+						display_progress( True, "Saving transformed background into \"%s\" ...", realfilename);
+    					LOCAL_DEBUG_OUT( "Saving transformed background into \"%s\" ...", realfilename);
+						success = save_asimage_to_file(	realfilename, scaled, 
+														"png", "9", NULL, 0, True);			   
+					}
+					if( scaled != tiled ) 
+						destroy_asimage( &scaled );
+					if( tiled != src_im ) 
+						destroy_asimage( &tiled );
+				}					   
+			}
+			safe_asimage_destroy( src_im );
+		}	 
+	}
+
+	if( !success )	           /* just tile as usuall */
+	{
+		display_progress( True, "Copying selected background \"%s\" into \"%s\" ...", data->text, realfilename);
+    	LOCAL_DEBUG_OUT( "Copying selected background \"%s\" into \"%s\" ...", data->text, realfilename);
+		success = (CopyFile (data->text, realfilename) == 0);
+	}
+
+    if ( success )
     {
 		++_as_background_change_count ;
 		if( Scr.CurrentDesk == 0 )
 		update_default_session ( Session, F_CHANGE_BACKGROUND );
-	change_desk_session (Session, Scr.CurrentDesk, realfilename, F_CHANGE_BACKGROUND);
+		change_desk_session (Session, Scr.CurrentDesk, realfilename, F_CHANGE_BACKGROUND);
 	}
 	free (realfilename);
 
@@ -1210,6 +1405,18 @@ void change_background_func_handler( FunctionData *data, ASEvent *event, int mod
 	--_as_config_change_recursion ;
 }
 
+
+void change_background_func_handler( FunctionData *data, ASEvent *event, int module )
+{
+	change_background_internal( data, False );
+}
+
+void change_back_foreign_func_handler( FunctionData *data, ASEvent *event, int module )
+{
+	change_background_internal( data, True );
+}	 
+
+
 void change_theme_func_handler( FunctionData *data, ASEvent *event, int module )
 {
 	++_as_config_change_recursion ;
@@ -1360,6 +1567,72 @@ void save_workspace_func_handler( FunctionData *data, ASEvent *event, int module
     save_aswindow_list( Scr.Windows, data->text?data->text:NULL );
 }
 
+Bool
+send_client_message_iter_func(void *data, void *aux_data)
+{
+    XClientMessageEvent *ev = (XClientMessageEvent *)aux_data;
+    ASWindow *asw = (ASWindow *)data ;
+	
+	ev->window = asw->w;
+    XSendEvent (dpy, asw->w, False, 0, (XEvent *) ev);
+
+    return True;
+}
+
+
+void signal_reload_GTKRC_file_handler( FunctionData *data, ASEvent *event, int module )
+{
+	XClientMessageEvent ev;
+	
+	memset( &ev, 0x00, sizeof(ev));
+	ev.type = ClientMessage;
+	ev.display = dpy ;
+	ev.message_type = _GTK_READ_RCFILES;
+	ev.format = 8;
+	
+	iterate_asbidirlist( Scr.Windows->clients, send_client_message_iter_func, &ev, NULL, False );
+}
+
+Bool
+send_kipc_client_message_iter_func(void *data, void *aux_data)
+{
+    XClientMessageEvent *ev = (XClientMessageEvent *)aux_data;
+    ASWindow *asw = (ASWindow *)data ;
+	
+	/* KDE not always sets this flag ??? */
+	if(  get_flags(asw->hints->protocols, AS_DoesKIPC ) )
+	{	
+		ev->window = asw->w; 
+    	XSendEvent (dpy, asw->w, False, 0, (XEvent *) ev);
+	}
+
+    return True;
+}
+
+void KIPC_sendMessageAll(KIPC_Message msg, int data)
+{
+	XClientMessageEvent ev;
+	
+	memset( &ev, 0x00, sizeof(ev));
+    ev.type = ClientMessage;
+    ev.display = dpy;
+    ev.message_type = _KIPC_COMM_ATOM;
+    ev.format = 32;
+    ev.data.l[0] = msg;
+    ev.data.l[1] = data;
+	
+	iterate_asbidirlist( Scr.Windows->clients, send_client_message_iter_func, &ev, NULL, False );
+	/*	iterate_asbidirlist( Scr.Windows->clients, send_kipc_client_message_iter_func, &ev, NULL, False ); */
+}
+
+
+
+void KIPC_send_message_all_handler( FunctionData *data, ASEvent *event, int module )
+{
+	KIPC_sendMessageAll((KIPC_Message)data->func_val[0], data->func_val[1]);
+	
+}	 
+
 void refresh_func_handler( FunctionData *data, ASEvent *event, int module )
 {
     XSetWindowAttributes attributes;
@@ -1483,6 +1756,23 @@ void killmodule_func_handler( FunctionData *data, ASEvent *event, int module )
     KillModuleByName (data->text);
 }
 
+void killallmodules_func_handler( FunctionData *data, ASEvent *event, int module )
+{
+    KillAllModulesByName (data->text);
+}
+
+void restartmodule_func_handler( FunctionData *data, ASEvent *event, int module )
+{
+	char *cmd_line = GetModuleCmdLineByName(data->text);
+	KillModuleByName (data->text);	
+	if( cmd_line ) 
+	{
+	    UngrabEm ();
+    	ExecModule (cmd_line, event->client ? event->client->w : None, event->context);
+		free( cmd_line );
+	}	 
+}
+
 void popup_func_handler( FunctionData *data, ASEvent *event, int module )
 {
     run_menu( data->text?data->text:data->name, event->client?event->client->w:None );
@@ -1498,10 +1788,26 @@ void windowlist_func_handler( FunctionData *data, ASEvent *event, int module )
 #ifndef NO_WINDOWLIST
     MenuData *md =  make_desk_winlist_menu( Scr.Windows, data->text == NULL ? event->scr->CurrentDesk: data->func_val[0], Scr.Feel.winlist_sort_order, False );
     if( md != NULL )
-	run_menu_data( md );
+	{	
+		run_menu_data( md );
+		destroy_menu_data( &md );
+	}	
 #endif /* ! NO_WINDOWLIST */
 }
 
+void modulelist_func_handler( FunctionData *data, ASEvent *event, int module )
+{
+    MenuData *md =  (data->func == F_STOPMODULELIST)?
+						make_stop_module_menu(Scr.Feel.winlist_sort_order ):
+						make_restart_module_menu(Scr.Feel.winlist_sort_order );
+    if( md != NULL )
+	{	
+		run_menu_data( md );
+		destroy_menu_data( &md );
+	}	 
+
+}
+
 void quickrestart_func_handler( FunctionData *data, ASEvent *event, int module )
 {
     QuickRestart (data->text);
@@ -1517,10 +1823,10 @@ send_aswindow_data_iter_func(void *data, void *aux_data)
 
     SendConfig (module_id.id, M_CONFIGURE_WINDOW, asw);
 	/* always start with RES_CLASS to let module know if this is a DockApp or not */
-    SendString (module_id.id, M_RES_CLASS, asw->w, asw->frame, asw, asw->hints->res_class, asw->hints->names_encoding[asw->hints->res_class_idx]);
-    SendString (module_id.id, M_RES_NAME,  asw->w, asw->frame, asw, asw->hints->res_name, asw->hints->names_encoding[asw->hints->res_name_idx]);
-    SendString (module_id.id, M_ICON_NAME, asw->w, asw->frame, asw, asw->hints->icon_name, asw->hints->names_encoding[asw->hints->icon_name_idx]);
-    SendString (module_id.id, M_WINDOW_NAME, asw->w,asw->frame, asw, asw->hints->names[0], asw->hints->names_encoding[0]);
+    SendString (module_id.id, M_RES_CLASS, asw->w, asw->frame, asw, asw->hints->res_class, get_hint_name_encoding(asw->hints, asw->hints->res_class_idx));
+    SendString (module_id.id, M_RES_NAME,  asw->w, asw->frame, asw, asw->hints->res_name, get_hint_name_encoding(asw->hints, asw->hints->res_name_idx));
+    SendString (module_id.id, M_ICON_NAME, asw->w, asw->frame, asw, asw->hints->icon_name, get_hint_name_encoding(asw->hints, asw->hints->icon_name_idx));
+	SendString (module_id.id, M_WINDOW_NAME, asw->w,asw->frame, asw, asw->hints->names[0], get_hint_name_encoding(asw->hints, 0));
     return True;
 }
 
@@ -1587,6 +1893,7 @@ void screenshot_func_handler( FunctionData *data, ASEvent *event, int module )
 		if( save_asimage_to_file(realfilename, im, type, compress, NULL, 0, replace) )
 			show_warning( "screenshot saved as \"%s\"", realfilename );
 		free( realfilename ) ;
+		destroy_asimage( &im );
 	}
 }
 
@@ -1595,9 +1902,6 @@ void swallow_window_func_handler( FunctionData *data, ASEvent *event, int module
 	if( event->client ) 
 	{	
 		if( data->text ) 
-			
-			
-			
 			module = FindModuleByName( data->text );
 		SendPacket( module, M_SWALLOW_WINDOW, 2, event->client->w, event->client->frame);
  	}	
