@@ -1,4 +1,4 @@
-/*
+/* 
  * Copyright (c) 2002,2003 Sasha Vasko <sasha@aftercode.net>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -65,47 +65,51 @@ update_window_transparency( ASWindow *asw, Bool force )
 	int i ;
 	ASCanvas *changed_canvases[6] = {NULL, NULL, NULL, NULL, NULL, NULL };
 
-	for( i = 0 ; i < FRAME_PARTS ; ++i )
-		if( asw->frame_bars[i] )
-		{
-			update_astbar_transparency (asw->frame_bars[i], asw->frame_sides[od->tbar2canvas_xref[i]], force);
-			if( DoesBarNeedsRendering(asw->frame_bars[i]) )
+    if( !ASWIN_GET_FLAGS( asw, AS_Iconic ) )
+	{
+		for( i = 0 ; i < FRAME_PARTS ; ++i )
+			if( asw->frame_bars[i] )
 			{
-				ASCanvas *c = asw->frame_sides[od->tbar2canvas_xref[i]];
-				changed_canvases[od->tbar2canvas_xref[i]] = c ;
-				render_astbar( asw->frame_bars[i], c );
+				update_astbar_transparency (asw->frame_bars[i], asw->frame_sides[od->tbar2canvas_xref[i]], force);
+				if( DoesBarNeedsRendering(asw->frame_bars[i]) )
+				{
+					ASCanvas *c = asw->frame_sides[od->tbar2canvas_xref[i]];
+					changed_canvases[od->tbar2canvas_xref[i]] = c ;
+					render_astbar( asw->frame_bars[i], c );
+				}
+			}
+
+		if( asw->tbar )
+		{
+			update_astbar_transparency (asw->tbar, asw->frame_sides[od->tbar_side], force);
+			if( DoesBarNeedsRendering(asw->tbar) )
+			{
+				changed_canvases[od->tbar_side] = asw->frame_sides[od->tbar_side] ;
+				render_astbar( asw->tbar, asw->frame_sides[od->tbar_side] );
 			}
 		}
-
-	if( asw->tbar )
+	}else
 	{
-		update_astbar_transparency (asw->tbar, asw->frame_sides[od->tbar_side], force);
-		if( DoesBarNeedsRendering(asw->tbar) )
+		if( asw->icon_button )
 		{
-			changed_canvases[od->tbar_side] = asw->frame_sides[od->tbar_side] ;
-			render_astbar( asw->tbar, asw->frame_sides[od->tbar_side] );
-		}
-	}
-
-	if( asw->icon_button )
-	{
-		update_astbar_transparency (asw->icon_button, asw->icon_canvas, force);
-		if( DoesBarNeedsRendering(asw->icon_button) )
-		{
-			changed_canvases[4] = asw->icon_canvas ;
-			render_astbar(asw->icon_button, asw->icon_canvas);
-		}
-	}
-	if( asw->icon_title )
-	{
-		update_astbar_transparency (asw->icon_title, asw->icon_title_canvas?asw->icon_title_canvas:asw->icon_canvas, force);
-		if( DoesBarNeedsRendering( asw->icon_title ) )
-		{
-			if( asw->icon_title_canvas != NULL && asw->icon_title_canvas != asw->icon_canvas )
-				changed_canvases[5] = asw->icon_title_canvas ;
-			else
+			update_astbar_transparency (asw->icon_button, asw->icon_canvas, force);
+			if( DoesBarNeedsRendering(asw->icon_button) )
+			{
 				changed_canvases[4] = asw->icon_canvas ;
-			render_astbar (asw->icon_title, asw->icon_title_canvas?asw->icon_title_canvas:asw->icon_canvas );
+				render_astbar(asw->icon_button, asw->icon_canvas);
+			}
+		}
+		if( asw->icon_title )
+		{
+			update_astbar_transparency (asw->icon_title, asw->icon_title_canvas?asw->icon_title_canvas:asw->icon_canvas, force);
+			if( DoesBarNeedsRendering( asw->icon_title ) )
+			{
+				if( asw->icon_title_canvas != NULL && asw->icon_title_canvas != asw->icon_canvas )
+					changed_canvases[5] = asw->icon_title_canvas ;
+				else
+					changed_canvases[4] = asw->icon_canvas ;
+				render_astbar (asw->icon_title, asw->icon_title_canvas?asw->icon_title_canvas:asw->icon_canvas );
+			}
 		}
 	}
 	for( i = 0 ; i < 6 ; ++i )
@@ -320,6 +324,8 @@ move_resize_longbar( ASTBarData *bar, ASCanvas *canvas, ASOrientation *od,
     unsigned int h = bar->height;
     int bar_size ;
 
+	LOCAL_DEBUG_OUT( "normal_offset = %d, normal_length = %d, corner_sizes = %d,%d, vert = %d, force = %d", 
+					 normal_offset, normal_length, corner_size1, corner_size2, vertical, force_render );
     /* swapping bar height in case of vertical orientation of the etire window: */
     *(od->in_width) = w ;
     *(od->in_height) = h ;
@@ -369,7 +375,8 @@ condense_tbar( ASTBarData *tbar, unsigned int max_size, unsigned int *off1, unsi
                         *off1 /= 2 ;
                         *off2 = *off1 ;
                     }    
-                }
+                }else
+					*off2 = max_size - condensed ;
             }else 
 				condensed = max_size ;
         }else
@@ -385,7 +392,8 @@ condense_tbar( ASTBarData *tbar, unsigned int max_size, unsigned int *off1, unsi
                         *off1 /= 2 ;
                         *off2 = *off1 ;
                     }    
-                }
+                }else
+					*off2 = max_size - condensed ;
             }else 
 				condensed = max_size ;
         }    
@@ -428,7 +436,7 @@ move_resize_frame_bars( ASWindow *asw, int side, ASOrientation *od, unsigned int
             set_flags( canvas->state, CANVAS_FORCE_MASK );
 
 		/* title always considered a "horizontal bar" */
-        if( move_resize_longbar( title, canvas, od, 0, tbar_width, tbar_offset1, tbar_offset2, False, force_render ) )
+		if( move_resize_longbar( title, canvas, od, 0, normal_width, tbar_offset1, tbar_offset2, False, force_render ) )
             rendered = True;
         tbar_size = *(od->in_height);
     }
@@ -467,6 +475,31 @@ move_resize_frame_bars( ASWindow *asw, int side, ASOrientation *od, unsigned int
     return rendered;
 }
 
+static void
+resize_frame_subwindows( ASWindow *asw, ASOrientation *od, unsigned int frame_win_width, unsigned int frame_win_height ) 
+{
+    register unsigned int *frame_size = &(asw->status->frame_size[0]) ;
+    unsigned int normal_width, normal_height ;
+
+	if( od == NULL ) 
+		od = get_orientation_data(asw);
+
+    *(od->in_width)  = asw->frame_canvas->width ;
+    *(od->in_height) = asw->frame_canvas->height ;
+    normal_width  = *(od->out_width)  ;
+    normal_height = *(od->out_height) ;
+				
+	resize_canvases( asw, od, normal_width, normal_height, frame_size );
+    if( !ASWIN_GET_FLAGS(asw, AS_Shaded ) )  /* leave shaded client alone ! */
+    	moveresize_canvas( asw->client_canvas,
+                           frame_size[FR_W],
+                           frame_size[FR_N],
+                           (int)frame_win_width-(int)(frame_size[FR_W]+frame_size[FR_E]),
+                           (int)frame_win_height-(int)(frame_size[FR_N]+frame_size[FR_S]));
+}
+
+
+
 Bool
 apply_window_status_size(register ASWindow *asw, ASOrientation *od)
 {
@@ -502,6 +535,10 @@ LOCAL_DEBUG_OUT( "**CONFG Client(%lx(%s))->status(%ux%u%+d%+d,%s,%s(%d>-%d))",
 					asw->frame_canvas->root_y != asw->status->y ||
 					asw->frame_canvas->width != new_width ||
 					asw->frame_canvas->height != new_height );
+
+		if( step_size <= 0 )
+			resize_frame_subwindows( asw, od, new_width, new_height ); 
+
         moveresize_canvas(  asw->frame_canvas, asw->status->x, asw->status->y,
                             new_width, new_height );
     }
@@ -629,6 +666,22 @@ check_frame_side_config( ASWindow *asw, Window w, ASOrientation *od )
 	return found;
 }
 
+static void 
+move_shading_frame( ASWindow *asw, ASOrientation *od, int step_size )
+{
+	if( asw->frame_sides[od->sbar_side] )
+	{
+		XRaiseWindow( dpy, asw->frame_sides[od->sbar_side]->w );
+		if( ASWIN_HFLAGS(asw, AS_VerticalTitle) )
+			move_canvas( asw->frame_sides[od->sbar_side], step_size - asw->frame_sides[od->sbar_side]->width, 0 );
+		else
+			move_canvas( asw->frame_sides[od->sbar_side], 0, step_size - asw->frame_sides[od->sbar_side]->height );
+	}
+	if( asw->frame_sides[od->tbar_side] )
+		XRaiseWindow( dpy, asw->frame_sides[od->tbar_side]->w );
+}
+
+
 /* this gets called when StructureNotify/SubstractureNotify arrives : */
 void
 on_window_moveresize( ASWindow *asw, Window w )
@@ -673,29 +726,36 @@ LOCAL_DEBUG_OUT( "changes=0x%X", changes );
         {
             register unsigned int *frame_size = &(asw->status->frame_size[0]) ;
             int step_size = make_shade_animation_step( asw, od );
-			LOCAL_DEBUG_OUT( "setp_size = %d" );
+			LOCAL_DEBUG_OUT( "step_size = %d", step_size );
             if( step_size <= 0 )  /* don't moveresize client window while shading !!!! */
             {
-                resize_canvases( asw, od, normal_width, normal_height, frame_size );
-                if( !ASWIN_GET_FLAGS(asw, AS_Shaded ) )  /* leave shaded client alone ! */
-                    moveresize_canvas( asw->client_canvas,
-                                    frame_size[FR_W],
-                                    frame_size[FR_N],
-                                    asw->frame_canvas->width-(frame_size[FR_W]+frame_size[FR_E]),
-                                    asw->frame_canvas->height-(frame_size[FR_N]+frame_size[FR_S]));
-            }else
+				resize_frame_subwindows( asw, od, 	asw->frame_canvas->width,
+								  					asw->frame_canvas->height ); 					
+	        }else
 			{ 
 				if( normal_height != step_size )
             	{
-                	sleep_a_millisec(10);
-                	/* we get smoother animation if we move decoration ahead of actually
-                 	* resizing frame window : */
-                	resize_canvases( asw, od, normal_width, step_size, frame_size );
                 	*(od->in_width)=normal_width ;
                 	*(od->in_height)=step_size ;
+			     	
+					if( normal_height < step_size )           	
+					{	
+						/* we get smoother animation if we move decoration ahead of actually
+                 		 * resizing frame window : */
+ 						move_shading_frame( asw, od, step_size );
+						ASSync( False );
+                		sleep_a_millisec(10);
 	/*LOCAL_DEBUG_OUT( "**SHADE Client(%lx(%s))->(%d>-%d)", asw->w, ASWIN_NAME(asw)?ASWIN_NAME(asw):"noname", asw->shading_steps, step_size );*/
-                	resize_canvas( asw->frame_canvas, *(od->out_width), *(od->out_height));
-                	ASSync(False);
+                		resize_canvas( asw->frame_canvas, *(od->out_width), *(od->out_height));
+                		ASSync(False);
+					}else
+					{	
+ 						sleep_a_millisec(10);
+                		resize_canvas( asw->frame_canvas, *(od->out_width), *(od->out_height));
+                		ASSync(False);
+						move_shading_frame( asw, od, step_size );
+						ASSync( False );
+					}	
             	}else
 				{  /* probably just the change of the look - titlebar may need to be resized */
 					resize_canvases( asw, od, normal_width, normal_height, frame_size );
@@ -782,10 +842,12 @@ on_window_title_changed( ASWindow *asw, Bool update_display )
 {
     if( AS_ASSERT(asw) )
         return ;
+    if( is_output_level_under_threshold(OUTPUT_LEVEL_HINTS) )
+		print_clean_hints( NULL, NULL, asw->hints );
     if( asw->tbar )
     {
         ASCanvas *canvas = ASWIN_HFLAGS(asw, AS_VerticalTitle)?asw->frame_sides[FR_W]:asw->frame_sides[FR_N];
-        if( change_astbar_first_label( asw->tbar, asw->hints->names[0], asw->hints->names_encoding[0] ) )
+        if( change_astbar_first_label( asw->tbar, ASWIN_NAME(asw), ASWIN_NAME_ENCODING(asw) ) )
             if( canvas && update_display )
             {
 				invalidate_canvas_save( canvas );
@@ -806,9 +868,10 @@ on_window_title_changed( ASWindow *asw, Bool update_display )
 					SetShape( asw, 0 );
             }
     }
-    if( asw->icon_title )
+    LOCAL_DEBUG_OUT( "icon_title = %p, icon_name = \"%s\"", asw->icon_title, ASWIN_ICON_NAME(asw)?ASWIN_ICON_NAME(asw):"(null)" );
+	if( asw->icon_title )
     {
-        if( change_astbar_first_label( asw->icon_title, ASWIN_ICON_NAME(asw), (asw->hints->icon_name_idx<0)?AS_Text_ASCII:asw->hints->names_encoding[asw->hints->icon_name_idx] ) )
+        if( change_astbar_first_label( asw->icon_title, ASWIN_ICON_NAME(asw), ASWIN_ICON_NAME_ENC(asw) ) )
             on_icon_changed( asw );
     }
 }
@@ -822,12 +885,13 @@ on_window_hints_changed( ASWindow *asw )
 
     if( AS_ASSERT(asw) )
         return ;
-
+    if( ASWIN_GET_FLAGS( asw, AS_Dead ) )
+		return;
     if( !collect_hints( ASDefaultScr, asw->w, HINT_ANY, &raw_hints ) )
         return ;
     if( is_output_level_under_threshold(OUTPUT_LEVEL_HINTS) )
         print_hints( NULL, NULL, &raw_hints );
-    hints = merge_hints( &raw_hints, Database, NULL, Scr.Look.supported_hints, HINT_ANY, NULL );
+    hints = merge_hints( &raw_hints, Database, NULL, Scr.Look.supported_hints, HINT_ANY, NULL, asw->w );
     destroy_raw_hints( &raw_hints, True );
     if( hints )
     {
@@ -861,10 +925,14 @@ on_window_hints_changed( ASWindow *asw )
 	if( hints2decorations( asw, old_hints ) )
 	    on_window_status_changed( asw, False, True );
 
-	if( mystrcmp( old_hints->names[0], hints->names[0] ) != 0 )
-	    broadcast_window_name( asw );
-	if( mystrcmp( old_hints->res_name, hints->res_name ) != 0 )
+	if( mystrcmp( old_hints->res_name, hints->res_name ) != 0 || 
+		mystrcmp( old_hints->res_class, hints->res_class ) != 0)
   		broadcast_res_names( asw );
+	if( mystrcmp( old_hints->names[0], hints->names[0] ) != 0 )
+	{	
+	    broadcast_window_name( asw );
+		set_flags( asw->internal_flags, ASWF_NameChanged );
+	}
 	if( mystrcmp( old_hints->icon_name, hints->icon_name ) != 0 )
 	    broadcast_icon_name( asw );
 
@@ -880,7 +948,9 @@ update_window_tbar_size(	ASWindow *asw )
         unsigned int tbar_width = 0 ;
         unsigned int tbar_height = 0 ;
         int x_offset = 0, y_offset = 0 ;
-        if( ASWIN_HFLAGS(asw, AS_VerticalTitle) )
+        LOCAL_DEBUG_OUT( "IsVertical = %lX", ASWIN_HFLAGS(asw, AS_VerticalTitle) );
+		
+		if( ASWIN_HFLAGS(asw, AS_VerticalTitle) )
         {
             tbar_size = calculate_astbar_width( asw->tbar );
             tbar_width = tbar_size ; 
@@ -1071,7 +1141,9 @@ LOCAL_DEBUG_OUT( "status geometry = %dx%d%+d%+d", asw->status->width, asw->statu
 
     /* now we need to move/resize our frame window */
     if( !apply_window_status_size(asw, od) )
-		broadcast_config (M_CONFIGURE_WINDOW, asw);  /* must enforce status change propagation */
+		changed = True ;
+	if( changed )
+		broadcast_config (M_STATUS_CHANGE, asw);  /* must enforce status change propagation */
     if( !ASWIN_GET_FLAGS(asw, AS_Dead) )
         set_client_state( asw->w, asw->status );
 }
@@ -1126,20 +1198,6 @@ LOCAL_DEBUG_CALLER_OUT( "(%p,%s focused)", asw, focused?"":"not" );
 			LOCAL_DEBUG_OUT( "canvas save swapped for side %d", i );
 			update_canvas = NULL ;
 		}
-#if 0				  
-		if( focused )
-		{
-			save_canvas(update_canvas);
-		}else
-		{
-			if( restore_canvas(asw->frame_sides[i]) )
-			{
-				LOCAL_DEBUG_OUT( "canvas restored for side %d", i );
-				update_canvas = NULL ;
-			}
-		}
-#endif
-
 		if( i == od->tbar_side )
 			set_astbar_focused( asw->tbar, update_canvas, focused );
 
@@ -1155,7 +1213,7 @@ LOCAL_DEBUG_CALLER_OUT( "(%p,%s focused)", asw, focused?"":"not" );
         asw->internal->on_hilite_changed( asw->internal, NULL, focused );
     if( ASWIN_GET_FLAGS( asw, AS_ShapedDecor ) )
         SetShape( asw, 0 );
-    if(!ASWIN_GET_FLAGS(asw, AS_Iconic))
+    if(ASWIN_GET_FLAGS(asw, AS_Iconic))
 	{	
         set_astbar_focused( asw->icon_button, asw->icon_canvas, focused );
         set_astbar_focused( asw->icon_title, asw->icon_title_canvas, focused );
@@ -1187,7 +1245,9 @@ LOCAL_DEBUG_CALLER_OUT( "(%p,%s)", asw, context2text(pressed_context));
         /* now posting all the changes on display :*/
         for( i = FRAME_SIDES; --i >= 0; )
             if( is_canvas_dirty(asw->frame_sides[i]) )
+			{
                 update_canvas_display( asw->frame_sides[i] );
+			}
         if( asw->internal && asw->internal->on_pressure_changed )
             asw->internal->on_pressure_changed( asw->internal, pressed_context&C_CLIENT );
     }else /* Iconic !!! */
@@ -1207,12 +1267,12 @@ LOCAL_DEBUG_CALLER_OUT( "(%p,%s)", asw, context2text(pressed_context));
 void
 save_aswindow_anchor( ASWindow *asw, Bool hor, Bool vert )
 {
-    if( hor && asw->saved_anchor.width == 0 )
+    if( hor && asw->saved_anchor.width == 0 && asw->status->width < Scr.MyDisplayWidth )
     {
         asw->saved_anchor.x = asw->anchor.x ;
         asw->saved_anchor.width = asw->anchor.width ;
     }
-    if( vert && asw->saved_anchor.height == 0 )
+    if( vert && asw->saved_anchor.height == 0 && asw->status->height < Scr.MyDisplayHeight  )
     {
         asw->saved_anchor.y = asw->anchor.y ;
         asw->saved_anchor.height = asw->anchor.height ;
@@ -1222,6 +1282,7 @@ save_aswindow_anchor( ASWindow *asw, Bool hor, Bool vert )
 void
 moveresize_aswindow_wm( ASWindow *asw, int x, int y, unsigned int width, unsigned int height, Bool save_anchor )
 {
+	LOCAL_DEBUG_CALLER_OUT( "asw(%p)->geom(%dx%d%+d%+d)->save_anchor(%d)", asw, x, y, width, height, save_anchor );
     if( !AS_ASSERT(asw) )
     {
         ASStatusHints scratch_status  = *(asw->status);
@@ -1307,52 +1368,14 @@ init_aswindow_status( ASWindow *t, ASStatusHints *status )
 		t->status->viewport_x = Scr.Vx ;
 		t->status->viewport_y = Scr.Vy ;
 	}
-	if (!get_flags (t->status->flags, AS_Sticky))
+	if (!get_flags (t->status->flags, AS_Sticky) && 
+		!ASWIN_HFLAGS(t, AS_UseCurrentViewport))
 	{
 		t->status->x -= t->status->viewport_x;
 		t->status->y -= t->status->viewport_y;
 	}
 	LOCAL_DEBUG_OUT( "status->pos = %+d%+d, Scr.Vpos = %+d%+d", t->status->x, t->status->y, Scr.Vx, Scr.Vy );
-
-    if( !get_flags(AfterStepState, ASS_NormalOperation) )
-    {
-        int min_x, min_y, max_x, max_y ;
-        int margin = Scr.MyDisplayWidth>>5 ;
-        if (!ASWIN_GET_FLAGS( t, AS_Sticky ))
-        {
-            min_x = -Scr.Vx ;
-            max_x = Scr.VxMax+Scr.MyDisplayWidth ;
-            min_y = -Scr.Vy ;
-            max_y = Scr.VyMax+Scr.MyDisplayHeight ;
-        }else
-        {
-            min_x = 0 ;
-            max_x = Scr.MyDisplayWidth ;
-            min_y = 0 ;
-            max_y = Scr.MyDisplayHeight ;
-        }
-        /* we have to make sure that window is visible !!!! */
-		LOCAL_DEBUG_OUT( "x_range(%d,%d), y_range(%d,%d), margin = %d", min_x, max_x, min_y, max_y, margin );
-        if( (int)t->status->x + (int)t->status->width < min_x + margin )
-            t->status->x = min_x + margin - (int)t->status->width ;
-        else if( (int)t->status->x > max_x-margin )
-            t->status->x = max_x - margin ;
-        if( (int)t->status->y + (int)t->status->height < min_y + margin )
-            t->status->y = min_y + margin - (int)t->status->height ;
-        else if( (int)t->status->y > max_y-margin )
-            t->status->y = max_y - margin ;
-
-		LOCAL_DEBUG_OUT( "status->pos = %+d%+d, Scr.Vpos = %+d%+d", t->status->x, t->status->y, Scr.Vx, Scr.Vy );
-
-        set_flags( t->status->flags, AS_Position );
-
-    }else if( get_flags(Scr.Feel.flags, NoPPosition))
-	{ 
-      	if( !get_flags( t->hints->flags, AS_Transient ) && 
-		    !get_flags( status->flags, AS_StartPositionUser ) )
-	        clear_flags( t->status->flags, AS_Position );
-	}
-    
+	
 	/* TODO: AS_Iconic */
 	if( !ASWIN_GET_FLAGS(t, AS_StartLayer ) )
         ASWIN_LAYER(t) = AS_LayerNormal ;
@@ -1361,39 +1384,118 @@ init_aswindow_status( ASWindow *t, ASStatusHints *status )
     else if( ASWIN_LAYER(t) > AS_LayerHighest )
         ASWIN_LAYER(t) = AS_LayerHighest ;
 
-    if( get_flags( status->flags, AS_MaximizedX|AS_MaximizedY ))
-        pending_placement = True ;
-    else if( !get_flags( t->status->flags, AS_Position ))
-    {
-        if( ! get_flags( t->status->flags, AS_StartsIconic ) )
-        {
-            int x = -1, y = -1;
-			pending_placement = True ;
-            ASQueryPointerRootXY( &x, &y );
-	        if( get_flags( t->hints->flags, AS_Transient|AS_ShortLived ) )
+	if( get_flags( t->status->flags, AS_Fullscreen ))
+	{
+        t->status->width = Scr.MyDisplayWidth ;
+        t->status->height = Scr.MyDisplayHeight ;
+        t->status->x = 0 ;
+        t->status->y = 0 ;
+	}else
+	{	
+		if( get_flags (t->hints->flags, AS_MinSize) ) 
+		{	
+			int width = t->status->width ;
+			int height = t->status->height ;
+			if( (!get_flags (t->status->flags, AS_StartSizeUser) && width < t->hints->min_width) || width == 1 )
+				width = min( t->hints->min_width, Scr.MyDisplayWidth );
+			if( (!get_flags (t->status->flags, AS_StartSizeUser) && height < t->hints->min_height) || height == 1 )
+				width = min( t->hints->min_height, Scr.MyDisplayHeight );
+			if( width != t->status->width || height != t->status->height ) 
 			{
-				x -= (int)t->status->width / 2 ;
-				y -= (int)t->status->height / 2 ;
-				set_flags( t->status->flags, AS_Position );
-				pending_placement = False ;
-			}
-
-			if( x + (int)t->status->width > (int)Scr.MyDisplayWidth )
-				x = (int)Scr.MyDisplayWidth - (int)t->status->width ;
-			if( x < 0 )
-				x = 0 ;
-
-			if( y + (int)t->status->height > (int) Scr.MyDisplayHeight )
-				y = Scr.MyDisplayHeight - (int)t->status->height ;
-			if( y < 0 ) 
-				y = 0 ;
-
-            t->status->x = x ;
-            t->status->y = y ;
+				int dx = 0, dy = 0 ;
+				if( t->hints->gravity == EastGravity || 
+					t->hints->gravity == SouthEastGravity || 
+					t->hints->gravity == NorthEastGravity ||
+					t->hints->gravity == CenterGravity )
+					dx = (int)(t->status->width) - width ; 
+				if( t->hints->gravity == SouthGravity || 
+					t->hints->gravity == SouthEastGravity || 
+					t->hints->gravity == SouthWestGravity ||
+					t->hints->gravity == CenterGravity)
+					dy = (int)(t->status->height) - height ; 
+				if( t->hints->gravity == EastGravity || t->hints->gravity == CenterGravity ) 
+					dx = dx/2 ;
+				if( t->hints->gravity == SouthGravity || t->hints->gravity == CenterGravity ) 
+					dy = dy/2 ;
+				t->status->x += dx ;			
+				t->status->y += dy ;			   
+				t->status->width = width ;			   
+				t->status->height = height ;			   
+				XResizeWindow( dpy, t->w, width, height );
+			}	 
+		}
+    	if( !get_flags(AfterStepState, ASS_NormalOperation) )
+    	{
+        	int min_x, min_y, max_x, max_y ;
+        	int margin = Scr.MyDisplayWidth>>5 ;
+        	if (!ASWIN_GET_FLAGS( t, AS_Sticky ))
+        	{
+            	min_x = -Scr.Vx ;
+            	max_x = Scr.VxMax+Scr.MyDisplayWidth ;
+            	min_y = -Scr.Vy ;
+            	max_y = Scr.VyMax+Scr.MyDisplayHeight ;
+        	}else
+        	{
+            	min_x = 0 ;
+            	max_x = Scr.MyDisplayWidth ;
+            	min_y = 0 ;
+            	max_y = Scr.MyDisplayHeight ;
+        	}
+        	/* we have to make sure that window is visible !!!! */
+			LOCAL_DEBUG_OUT( "x_range(%d,%d), y_range(%d,%d), margin = %d", min_x, max_x, min_y, max_y, margin );
+        	if( (int)t->status->x + (int)t->status->width < min_x + margin )
+            	t->status->x = min_x + margin - (int)t->status->width ;
+        	else if( (int)t->status->x > max_x-margin )
+            	t->status->x = max_x - margin ;
+        	if( (int)t->status->y + (int)t->status->height < min_y + margin )
+            	t->status->y = min_y + margin - (int)t->status->height ;
+        	else if( (int)t->status->y > max_y-margin )
+            	t->status->y = max_y - margin ;
 
 			LOCAL_DEBUG_OUT( "status->pos = %+d%+d, Scr.Vpos = %+d%+d", t->status->x, t->status->y, Scr.Vx, Scr.Vy );
-        }
-    }
+
+        	set_flags( t->status->flags, AS_Position );
+
+    	}else if( get_flags(Scr.Feel.flags, NoPPosition))
+		{ 
+      		if( !get_flags( t->hints->flags, AS_Transient ) && 
+		    	!get_flags( status->flags, AS_StartPositionUser ) )
+	        	clear_flags( t->status->flags, AS_Position );
+		}
+		 if( get_flags( status->flags, AS_MaximizedX|AS_MaximizedY ))
+        	pending_placement = True ;
+		else  if( !get_flags( t->status->flags, AS_Position ))
+    	{
+        	if( ! get_flags( t->status->flags, AS_StartsIconic ) )
+        	{
+            	int x = -1, y = -1;
+				pending_placement = True ;
+            	ASQueryPointerRootXY( &x, &y );
+	        	if( get_flags( t->hints->flags, AS_Transient|AS_ShortLived ) )
+				{
+					x -= (int)t->status->width / 2 ;
+					y -= (int)t->status->height / 2 ;
+					set_flags( t->status->flags, AS_Position );
+					pending_placement = False ;
+				}
+
+				if( x + (int)t->status->width > (int)Scr.MyDisplayWidth )
+					x = (int)Scr.MyDisplayWidth - (int)t->status->width ;
+				if( x < 0 )
+					x = 0 ;
+
+				if( y + (int)t->status->height > (int) Scr.MyDisplayHeight )
+					y = Scr.MyDisplayHeight - (int)t->status->height ;
+				if( y < 0 ) 
+					y = 0 ;
+
+            	t->status->x = x ;
+            	t->status->y = y ;
+
+				LOCAL_DEBUG_OUT( "status->pos = %+d%+d, Scr.Vpos = %+d%+d", t->status->x, t->status->y, Scr.Vx, Scr.Vy );
+        	}
+    	}
+	}		
 
     if( is_output_level_under_threshold(OUTPUT_LEVEL_HINTS) )
         print_status_hints( NULL, NULL, t->status );
@@ -1412,6 +1514,7 @@ init_aswindow_status( ASWindow *t, ASStatusHints *status )
         validate_window_anchor( t, &(t->anchor) );    
         anchor2status ( t->status, t->hints, &(t->anchor));
     }    
+
     return pending_placement;
 }
 
@@ -1437,7 +1540,7 @@ complete_wm_state_transition( ASWindow *asw, int state )
 }
 
 Bool
-set_window_wm_state( ASWindow *asw, Bool iconify )
+set_window_wm_state( ASWindow *asw, Bool iconify, Bool force_unmapped )
 {
     XWindowAttributes attr;
 
@@ -1469,10 +1572,6 @@ LOCAL_DEBUG_CALLER_OUT( "client = %p, iconify = %d", asw, iconify );
         else if( asw->icon_title_canvas )
             asw->status->icon_window = asw->icon_title_canvas->w ;
 
-		LOCAL_DEBUG_OUT( "hilited == %p", Scr.Windows->hilited );
-    	if( Scr.Windows->hilited == asw )
-			hide_hilite();
-
         if (get_flags(Scr.Feel.flags, ClickToFocus) || get_flags(Scr.Feel.flags, SloppyFocus))
         {
             if (asw == Scr.Windows->focused)
@@ -1494,8 +1593,14 @@ LOCAL_DEBUG_OUT( "unmaping client window 0x%lX", (unsigned long)asw->w );
                     complete_wm_state_transition( asw, IconicState );
                 }
             }
+			ASSync(False);
         }
-        /* finally mapping the icon windows : */
+		
+		LOCAL_DEBUG_OUT( "hilited == %p", Scr.Windows->hilited );
+    	if( Scr.Windows->hilited == asw )
+			hide_hilite();
+        
+		/* finally mapping the icon windows : */
         add_iconbox_icon( asw );
         restack_window( asw, None, Below );
         map_canvas_window(asw->icon_canvas, True );
@@ -1516,20 +1621,21 @@ LOCAL_DEBUG_OUT( "updating status to iconic for client %p(\"%s\")", asw, ASWIN_N
             if( asw->icon_canvas != asw->icon_title_canvas )
                 unmap_canvas_window(asw->icon_title_canvas );
 
-            ASWIN_DESK(asw) = get_flags(Scr.Feel.flags, StubbornIcons)?asw->DeIconifyDesk:Scr.CurrentDesk;
-            quietly_reparent_aswindow( asw, (ASWIN_DESK(asw)==Scr.CurrentDesk)?Scr.Root:Scr.ServiceWin, True );
+			change_aswindow_desktop( asw, get_flags(Scr.Feel.flags, StubbornIcons)?asw->DeIconifyDesk:Scr.CurrentDesk, True );
         }
 
         asw->status->icon_window = None ;
 
-        if( !ASWIN_GET_FLAGS(asw, AS_Dead) )
+        if( !ASWIN_GET_FLAGS(asw, AS_Dead) && !force_unmapped )
         {
             if ( !XGetWindowAttributes (dpy, asw->w, &attr) )
                 ASWIN_SET_FLAGS(asw, AS_Dead);
-            else
+		}
+        if( !ASWIN_GET_FLAGS(asw, AS_Dead) )
+        {
             {
                 /* TODO: make sure that the window is on this screen */
-                if( attr.map_state == IsUnmapped )
+                if( attr.map_state == IsUnmapped || force_unmapped)
                     XMapRaised (dpy, asw->w);
                 else
                 {
@@ -1618,7 +1724,7 @@ make_aswindow_visible( ASWindow *asw, Bool deiconify )
     {
         if( deiconify )
         {
-			set_window_wm_state( asw, False );
+			set_window_wm_state( asw, False, False );
 		}
     }
 
@@ -1684,8 +1790,8 @@ make_aswindow_visible( ASWindow *asw, Bool deiconify )
         }else
         {
             on_window_moveresize( asw, asw->frame );
-            x = asw->frame_canvas->root_x;
-            y = asw->frame_canvas->root_y;
+            x = asw->client_canvas->root_x;
+            y = asw->client_canvas->root_y;
         }
 		LOCAL_DEBUG_OUT( "Warping pointer to : %+d%+d", x + Scr.Feel.Xzap, y + Scr.Feel.Yzap);
         XWarpPointer (dpy, None, Scr.Root, 0, 0, 0, 0, x + Scr.Feel.Xzap, y + Scr.Feel.Yzap);
@@ -1710,12 +1816,15 @@ LOCAL_DEBUG_OUT( "changing window's layer to %d", layer );
     }
 }
 
-void change_aswindow_desktop( ASWindow *asw, int new_desk )
+void change_aswindow_desktop( ASWindow *asw, int new_desk, Bool force )
 {
     int old_desk ;
     if( AS_ASSERT(asw) )
         return ;
-    if( ASWIN_DESK(asw) == new_desk || ASWIN_GET_FLAGS(asw, AS_Sticky))
+	if( ASWIN_GET_FLAGS(asw, AS_Sticky) ) 
+		new_desk = Scr.CurrentDesk ;
+    
+	if( !force && ASWIN_DESK(asw) == new_desk )
         return ;
 
     old_desk = ASWIN_DESK(asw) ;
@@ -1734,6 +1843,31 @@ void change_aswindow_desktop( ASWindow *asw, int new_desk )
     }
 }
 
+static Bool
+restore_anchor_x( ASWindow *asw )
+{
+	if( asw->saved_anchor.width > 0)
+    {
+    	asw->anchor.x = asw->saved_anchor.x ;
+        asw->anchor.width = asw->saved_anchor.width ;
+        asw->saved_anchor.width = 0 ;  /* invalidating saved anchor */
+		return True;
+	}
+	return False;
+}
+static Bool
+restore_anchor_y( ASWindow *asw )
+{
+    if( asw->saved_anchor.height > 0)
+    {
+        asw->anchor.y = asw->saved_anchor.y ;
+        asw->anchor.height = asw->saved_anchor.height ;
+        asw->saved_anchor.height = 0 ; /* invalidating saved anchor */
+        return True ;
+	}
+	return False;	  
+}	  
+
 void toggle_aswindow_status( ASWindow *asw, ASFlagType flags )
 {
     ASFlagType on_flags, off_flags ;
@@ -1742,9 +1876,13 @@ void toggle_aswindow_status( ASWindow *asw, ASFlagType flags )
 
     if( AS_ASSERT(asw) )
         return ;
+
     if( flags == 0 )
         return ;
 
+	if( get_flags( flags, AS_Fullscreen ) )
+		clear_flags( flags, AS_MaximizedX|AS_MaximizedY);
+
     on_flags = (~(asw->status->flags))&flags ;
     off_flags = (asw->status->flags)&(~flags) ;
     asw->status->flags = on_flags|off_flags ;
@@ -1779,32 +1917,56 @@ LOCAL_DEBUG_OUT( "flags = %lx, on_flags = %lx, off_flags = %lx", flags, on_flags
 		}
 	}
 
+	if( get_flags( flags, AS_Fullscreen) )
+	{
+		if( !ASWIN_GET_FLAGS( asw, AS_Fullscreen ) )			   
+		{	                   /* fullscreen->normal */
+			if( !ASWIN_GET_FLAGS( asw, AS_MaximizedX ) )			
+	            if( !restore_anchor_x( asw ) )
+				{
+					asw->anchor.width = Scr.MyDisplayWidth/3;
+					asw->anchor.x = (Scr.MyDisplayWidth - asw->anchor.width) / 2;
+				}
+
+	        if( !ASWIN_GET_FLAGS( asw, AS_MaximizedY ) )
+	            if(!restore_anchor_y( asw ))
+				{
+					asw->anchor.height = Scr.MyDisplayHeight/3;
+					asw->anchor.y = (Scr.MyDisplayHeight - asw->anchor.height) / 2;
+				}
+
+			reconfigured = True;
+			if( ASWIN_GET_FLAGS( asw, AS_MaximizedY|AS_MaximizedX ) )
+				need_placement = True ;	
+		}else
+		{ 	                   /* normal->fullscreen */
+			ASStatusHints scratch_status = *(asw->status) ;
+			scratch_status.x = 0 ;
+			scratch_status.y = 0 ;
+			scratch_status.width = Scr.MyDisplayWidth ;
+			scratch_status.height = Scr.MyDisplayHeight ;
+	 		
+			save_aswindow_anchor( asw, True, True );		   
+			status2anchor(&(asw->anchor), asw->hints, &scratch_status, Scr.VxMax, Scr.VyMax);
+			reconfigured = True ;
+		}
+	}
     if( get_flags( flags, AS_MaximizedX) )
     {
         if( !ASWIN_GET_FLAGS( asw, AS_MaximizedX ) )
-        {
-            if( asw->saved_anchor.width > 0)
-            {
-                asw->anchor.x = asw->saved_anchor.x ;
-                asw->anchor.width = asw->saved_anchor.width ;
-                asw->saved_anchor.width = 0 ;  /* invalidating saved anchor */
-                reconfigured = True ;
-            }
-        }else
+		{	
+            if( restore_anchor_x( asw ) ) 
+				reconfigured = True ;
+        }else if( !ASWIN_GET_FLAGS( asw, AS_Fullscreen ) )
             need_placement = True ;
     }
     if( get_flags( flags, AS_MaximizedY) )
     {
         if( !ASWIN_GET_FLAGS( asw, AS_MaximizedY ) )
         {
-            if( asw->saved_anchor.height > 0)
-            {
-                asw->anchor.y = asw->saved_anchor.y ;
-                asw->anchor.height = asw->saved_anchor.height ;
-                asw->saved_anchor.height = 0 ; /* invalidating saved anchor */
-                reconfigured = True ;
-            }
-        }else
+            if(restore_anchor_y( asw ))
+				reconfigured = True;
+        }else if( !ASWIN_GET_FLAGS( asw, AS_Fullscreen ) )
             need_placement = True ;
     }
 
@@ -1839,7 +2001,7 @@ LOCAL_DEBUG_OUT("current focused is %p, active is %p", Scr.Windows->focused, Scr
         {
 LOCAL_DEBUG_OUT( "Window is iconic - pending implementation%s","");
             if( deiconify )
-				set_window_wm_state( asw, False );
+				set_window_wm_state( asw, False, False );
             else
                 return False;
         }
